@@ -18,19 +18,20 @@ def main():
     common.setup_global_opts(parser)
     parser.add_argument("appid", nargs='*', help=_("applicationId with optional versionCode in the form APPID[:VERCODE]"))
     metadata.add_metadata_arguments(parser)
-    options = parser.parse_args()
-    common.options = options
-    pkgs = common.read_pkg_args(options.appid, True)
-    allapps = metadata.read_metadata(pkgs)
-    apps = common.read_app_args(options.appid, allapps, True)
-    common.read_config(options)
+    options = common.parse_args(parser)
+    apps = common.read_app_args(options.appid, allow_version_codes=True, sort_by_time=True)
+    common.read_config()
     srclib_dir = os.path.join('build', 'srclib')
     os.makedirs(srclib_dir, exist_ok=True)
     srclibpaths = []
     for appid, app in apps.items():
         vcs, _ignored = common.setup_vcs(app)
-        vcs.gotorevision('HEAD', refresh=False)
         for build in app.get('Builds', []):
+            vcs.gotorevision(build.commit, refresh=False)
+            if build.submodules:
+                vcs.initsubmodules()
+            else:
+                vcs.deinitsubmodules()
             for lib in build.srclibs:
                 srclibpaths.append(common.getsrclib(lib, srclib_dir, prepare=False, build=build))
     print('Set up srclibs:')
