@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+"""Build a package from source."""
 #
 # build.py - part of the FDroid server tools
 # Copyright (C) 2010-2014, Ciaran Gultnieks, ciaran@ciarang.com
@@ -59,15 +60,30 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
     Parameters
     ----------
     app
-        app metadata dict
+        The metadata of the app to build.
     build
+        The build of the app to build.
     vcs
-        version control system controller object
+        The version control system controller object of the app.
     build_dir
-        local source-code checkout of app
+        The local source-code checkout directory of the app.
     output_dir
-        target folder for the build result
+        The target folder for the build result.
+    log_dir
+        The directory in the VM where the build logs are getting stored.
     force
+        Don't refresh the already cloned repository and make the build stop on
+        exceptions.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.BuildException`
+        If Paramiko is not installed, a srclib directory or srclib metadata
+        file is unexpectedly missing, the build process in the VM failed or
+        output files of the build process are missing.
+    :exc:`~fdroidserver.exception.FDroidException`
+        If the Buildserver ID could not be obtained or copying a directory to
+        the server failed.
     """
     global buildserverid, ssh_channel
 
@@ -101,7 +117,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         # Open SSH connection...
         logging.info("Connecting to virtual machine...")
         sshs = paramiko.SSHClient()
-        sshs.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+        sshs.set_missing_host_key_policy(paramiko.AutoAddPolicy())  # nosec B507 only connects to local VM
         sshs.connect(sshinfo['hostname'], username=sshinfo['user'],
                      port=sshinfo['port'], timeout=300,
                      look_for_keys=False, key_filename=sshinfo['idfile'])
@@ -115,8 +131,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         # Put all the necessary files in place...
         ftp.chdir(homedir)
 
-        # Helper to copy the contents of a directory to the server...
         def send_dir(path):
+            """Copy the contents of a directory to the server."""
             logging.debug("rsyncing %s to %s" % (path, ftp.getcwd()))
             # TODO this should move to `vagrant rsync` from >= v1.5
             try:
@@ -232,6 +248,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             cmdline += ' --force --test'
         if options.verbose:
             cmdline += ' --verbose'
+        if options.refresh_scanner or config.get('refresh_scanner'):
+            cmdline += ' --refresh-scanner'
         if options.skipscan:
             cmdline += ' --skip-scan'
         if options.notarball:
@@ -250,13 +268,13 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
                 line = cmd_stdout.readline()
                 if line:
                     if options.verbose:
-                        logging.debug("buildserver > " + str(line, 'utf-8').rstrip())
+                        logging.debug("buildserver > " + str(line, 'utf-8', 'replace').rstrip())
                     output += line
                 else:
                     time.sleep(0.05)
             for line in cmd_stdout.readlines():
                 if options.verbose:
-                    logging.debug("buildserver > " + str(line, 'utf-8').rstrip())
+                    logging.debug("buildserver > " + str(line, 'utf-8', 'replace').rstrip())
                 output += line
         finally:
             cmd_stdout.close()
@@ -270,7 +288,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             else:
                 message = "Build.py failed on server for {0}:{1}"
             raise BuildException(message.format(app.id, build.versionName),
-                                 str(output, 'utf-8'))
+                                 str(output, 'utf-8', 'replace'))
 
         # Retreive logs...
         toolsversion_log = common.get_toolsversion_logname(app, build)
@@ -288,7 +306,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         else:
             ftp.chdir(posixpath.join(homedir, 'unsigned'))
         apkfile = common.get_release_filename(app, build)
-        tarball = common.getsrcname(app, build)
+        tarball = common.get_src_tarball_name(app.id, build.versionCode)
         try:
             ftp.get(apkfile, os.path.join(output_dir, apkfile))
             if not options.notarball:
@@ -296,7 +314,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         except Exception as exc:
             raise BuildException(
                 "Build failed for {0}:{1} - missing output files".format(
-                    app.id, build.versionName), str(output, 'utf-8')) from exc
+                    app.id, build.versionName), str(output, 'utf-8', 'replace')) from exc
         ftp.close()
 
     finally:
@@ -314,6 +332,15 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
 
 
 def force_gradle_build_tools(build_dir, build_tools):
+    """Manipulate build tools version used in top level gradle file.
+
+    Parameters
+    ----------
+    build_dir
+        The directory to start looking for gradle files.
+    build_tools
+        The build tools version that should be forced to use.
+    """
     for root, dirs, files in os.walk(build_dir):
         for filename in files:
             if not filename.endswith('.gradle'):
@@ -340,6 +367,31 @@ def get_metadata_from_apk(app, build, apkfile):
     """Get the required metadata from the built APK.
 
     VersionName is allowed to be a blank string, i.e. ''
+
+    Parameters
+    ----------
+    app
+        The app metadata used to build the APK.
+    build
+        The build that resulted in the APK.
+    apkfile
+        The path of the APK file.
+
+    Returns
+    -------
+    versionCode
+        The versionCode from the APK or from the metadata is build.novcheck is
+        set.
+    versionName
+        The versionName from the APK or from the metadata is build.novcheck is
+        set.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.BuildException`
+        If native code should have been built but was not packaged, no version
+        information or no package ID could be found or there is a mismatch
+        between the package ID in the metadata and the one found in the APK.
     """
     appid, versionCode, versionName = common.get_apk_id(apkfile)
     native_code = common.get_native_code(apkfile)
@@ -360,7 +412,56 @@ def get_metadata_from_apk(app, build, apkfile):
 
 
 def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, extlib_dir, tmp_dir, force, onserver, refresh):
-    """Do a build locally."""
+    """Do a build locally.
+
+    Parameters
+    ----------
+    app
+        The metadata of the app to build.
+    build
+        The build of the app to build.
+    vcs
+        The version control system controller object of the app.
+    build_dir
+        The local source-code checkout directory of the app.
+    output_dir
+        The target folder for the build result.
+    log_dir
+        The directory in the VM where the build logs are getting stored.
+    srclib_dir
+        The path to the srclibs directory, usually 'build/srclib'.
+    extlib_dir
+        The path to the extlibs directory, usually 'build/extlib'.
+    tmp_dir
+        The temporary directory for building the source tarball.
+    force
+        Don't refresh the already cloned repository and make the build stop on
+        exceptions.
+    onserver
+        Assume the build is happening inside the VM.
+    refresh
+        Enable fetching the latest refs from the VCS remote.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.BuildException`
+        If running a `sudo` command failed, locking the root account failed,
+        `sudo` couldn't be removed, cleaning the build environment failed,
+        skipping the scanning has been requested but `scandelete` is present,
+        errors occurred during scanning, running the `build` commands from the
+        metadata failed, building native code failed, building with the
+        specified build method failed, no output could be found with build
+        method `maven`, more or less than one APK were found with build method
+        `gradle`, less or more than one APKs match the `output` glob specified
+        in the metadata, running a `postbuild` command specified in the
+        metadata failed, the built APK is debuggable, the unsigned APK is not
+        at the expected location, the APK does not contain the expected
+        `versionName` and `versionCode` or undesired package names have been
+        found in the APK.
+    :exc:`~fdroidserver.exception.FDroidException`
+        If no Android NDK version could be found and the build isn't run in a
+        builder VM, the selected Android NDK is not a directory.
+    """
     ndk_path = build.ndk_path()
     if build.ndk or (build.buildjni and build.buildjni != ['no']):
         if not ndk_path:
@@ -518,7 +619,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
     if not options.notarball:
         # Build the source tarball right before we build the release...
         logging.info("Creating source tarball...")
-        tarname = common.getsrcname(app, build)
+        tarname = common.get_src_tarball_name(app.id, build.versionCode)
         tarball = tarfile.open(os.path.join(tmp_dir, tarname), "w:gz")
 
         def tarexc(t):
@@ -718,8 +819,10 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
                                  f"{app.id}:{build.versionName}", p.output)
 
     # Make sure it's not debuggable...
-    if common.is_apk_and_debuggable(src):
-        raise BuildException("APK is debuggable")
+    if common.is_debuggable_or_testOnly(src):
+        raise BuildException(
+            "%s: debuggable or testOnly set in AndroidManifest.xml" % src
+        )
 
     # By way of a sanity check, make sure the version and version
     # code in our new APK match what we expect...
@@ -762,23 +865,47 @@ def trybuild(app, build, build_dir, output_dir, log_dir, also_check_dir,
 
     Parameters
     ----------
+    app
+        The metadata of the app to build.
+    build
+        The build of the app to build.
+    build_dir
+        The local source-code checkout directory of the app.
     output_dir
-        The directory where the build output will go.
-        Usually this is the 'unsigned' directory.
+        The directory where the build output will go.  Usually this is the
+        'unsigned' directory.
+    log_dir
+        The directory in the VM where the build logs are getting stored.
+    also_check_dir
+        An additional location for checking if the build is necessary (usually
+        the archive repo).
+    srclib_dir
+        The path to the srclibs directory, usually 'build/srclib'.
+    extlib_dir
+        The path to the extlibs directory, usually 'build/extlib'.
+    tmp_dir
+        The temporary directory for building the source tarball of the app to
+        build.
     repo_dir
         The repo directory - used for checking if the build is necessary.
-    also_check_dir
-        An additional location for checking if the build
-        is necessary (usually the archive repo)
+    vcs
+        The version control system controller object of the app to build.
     test
-        True if building in test mode, in which case the build will
-        always happen, even if the output already exists. In test mode, the
-        output directory should be a temporary location, not any of the real
-        ones.
+        True if building in test mode, in which case the build will always
+        happen, even if the output already exists.  In test mode, the output
+        directory should be a temporary location, not any of the real ones.
+    server
+        Use buildserver VM for building.
+    force
+        Build app regardless of disabled state or scanner errors.
+    onserver
+        Assume the build is happening inside the VM.
+    refresh
+        Enable fetching the latest refs from the VCS remote.
 
     Returns
     -------
-    Boolean
+    status
         True if the build was done, False if it wasn't necessary.
     """
     dest_file = common.get_release_filename(app, build)
@@ -817,7 +944,13 @@ def trybuild(app, build, build_dir, output_dir, log_dir, also_check_dir,
 
 
 def force_halt_build(timeout):
-    """Halt the currently running Vagrant VM, to be called from a Timer."""
+    """Halt the currently running Vagrant VM, to be called from a Timer.
+
+    Parameters
+    ----------
+    timeout
+        The timeout in seconds.
+    """
     logging.error(_('Force halting build after {0} sec timeout!').format(timeout))
     timeout_event.set()
     if ssh_channel:
@@ -826,13 +959,24 @@ def force_halt_build(timeout):
     vm.destroy()
 
 
+def keep_when_not_allowed():
+    """Control if APKs signed by keys not in AllowedAPKSigningKeys are removed."""
+    return (
+        (options is not None and options.keep_when_not_allowed)
+        or (config is not None and config.get('keep_when_not_allowed'))
+        or common.default_config['keep_when_not_allowed']
+    )
+
+
 def parse_commandline():
     """Parse the command line.
 
     Returns
     -------
     options
+        The resulting options parsed from the command line arguments.
     parser
+        The argument parser.
     """
     parser = argparse.ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
@@ -845,8 +989,6 @@ def parse_commandline():
                         help=_("Test mode - put output in the tmp directory only, and always build, even if the output already exists."))
     parser.add_argument("--server", action="store_true", default=False,
                         help=_("Use build server"))
-    parser.add_argument("--reset-server", action="store_true", default=False,
-                        help=_("Reset and create a brand new build server, even if the existing one appears to be ok."))
     # this option is internal API for telling fdroid that
     # it's running inside a buildserver vm.
     parser.add_argument("--on-server", dest="onserver", action="store_true", default=False,
@@ -859,14 +1001,18 @@ def parse_commandline():
                         help=_("Don't create a source tarball, useful when testing a build"))
     parser.add_argument("--no-refresh", dest="refresh", action="store_false", default=True,
                         help=_("Don't refresh the repository, useful when testing a build with no internet connection"))
+    parser.add_argument("-r", "--refresh-scanner", dest="refresh_scanner", action="store_true", default=False,
+                        help=_("Refresh and cache scanner rules and signatures from the network"))
     parser.add_argument("-f", "--force", action="store_true", default=False,
                         help=_("Force build of disabled apps, and carries on regardless of scan problems. Only allowed in test mode."))
     parser.add_argument("-a", "--all", action="store_true", default=False,
                         help=_("Build all applications available"))
+    parser.add_argument("--keep-when-not-allowed", default=False, action="store_true",
+                        help=argparse.SUPPRESS)
     parser.add_argument("-w", "--wiki", default=False, action="store_true",
                         help=argparse.SUPPRESS)
     metadata.add_metadata_arguments(parser)
-    options = parser.parse_args()
+    options = common.parse_args(parser)
     metadata.warnings_action = options.W
 
     # Force --stop with --on-server to get correct exit code
@@ -888,7 +1034,22 @@ timeout_event = threading.Event()
 
 
 def main():
+    """Build a package from source.
 
+    The behaviour of this function is influenced by the configuration file as
+    well as command line parameters.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.FDroidException`
+        If more than one local metadata file has been found, no app metadata
+        has been found, there are no apps to process, downloading binaries for
+        checking the reproducibility of a built binary failed, the built binary
+        is different from supplied reference binary, the reference binary is
+        signed with a different signing key than expected, a VCS error occured
+        while building an app or a different error occured while building an
+        app.
+    """
     global options, config, buildserverid, fdroidserverid
 
     options, parser = parse_commandline()
@@ -913,12 +1074,10 @@ def main():
         if not options.appid and not options.all:
             parser.error("option %s: If you really want to build all the apps, use --all" % "all")
 
-    config = common.read_config(opts=options)
+    config = common.read_config()
 
     if config['build_server_always']:
         options.server = True
-    if options.reset_server and not options.server:
-        parser.error("option %s: Using --reset-server without --server makes no sense" % "reset-server")
 
     log_dir = 'logs'
     if not os.path.isdir(log_dir):
@@ -958,10 +1117,7 @@ def main():
     srclib_dir = os.path.join(build_dir, 'srclib')
     extlib_dir = os.path.join(build_dir, 'extlib')
 
-    # Read all app and srclib metadata
-    pkgs = common.read_pkg_args(options.appid, True)
-    allapps = metadata.read_metadata(pkgs, sort_by_time=True)
-    apps = common.read_app_args(options.appid, allapps, True)
+    apps = common.read_app_args(options.appid, allow_version_codes=True, sort_by_time=True)
 
     for appid, app in list(apps.items()):
         if (app.get('Disabled') and not options.force) or not app.get('RepoType') or not app.get('Builds', []):
@@ -1113,6 +1269,34 @@ def main():
                                              'supplied reference binary '
                                              'successfully')
 
+                            used_key = common.apk_signer_fingerprint(of)
+                            expected_keys = app['AllowedAPKSigningKeys']
+                            if used_key is None:
+                                logging.warn(_('reference binary missing '
+                                               'signature'))
+                            elif len(expected_keys) == 0:
+                                logging.warn(_('AllowedAPKSigningKeys missing '
+                                               'but reference binary supplied'))
+                            elif used_key not in expected_keys:
+                                if options.test or keep_when_not_allowed():
+                                    logging.warning(_('Keeping failed build "{apkfilename}"')
+                                                    .format(apkfilename=unsigned_apk))
+                                else:
+                                    logging.debug('removing %s', unsigned_apk)
+                                    os.remove(unsigned_apk)
+                                logging.debug('removing %s', of)
+                                os.remove(of)
+                                raise FDroidException('supplied reference '
+                                                      'binary signed with '
+                                                      '{signer} instead of '
+                                                      'with {expected}'.
+                                                      format(signer=used_key,
+                                                             expected=expected_keys))
+                            else:
+                                logging.info(_('supplied reference binary has '
+                                               'allowed signer {signer}').
+                                             format(signer=used_key))
+
                     build_succeeded_ids.append([app['id'], build.versionCode])
 
                     if not options.onserver:
@@ -1123,7 +1307,7 @@ def main():
                 logging.error("VCS error while building app %s: %s" % (
                     appid, reason))
                 if options.stop:
-                    logging.debug("Error encoutered, stopping by user request.")
+                    logging.debug("Error encountered, stopping by user request.")
                     common.force_exit(1)
                 failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
@@ -1144,7 +1328,7 @@ def main():
                     f.write(str(e))
                 logging.error("Could not build app %s: %s" % (appid, e))
                 if options.stop:
-                    logging.debug("Error encoutered, stopping by user request.")
+                    logging.debug("Error encountered, stopping by user request.")
                     common.force_exit(1)
                 failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
@@ -1157,7 +1341,7 @@ def main():
                 logging.error("Could not build app %s due to unknown error: %s" % (
                     appid, traceback.format_exc()))
                 if options.stop:
-                    logging.debug("Error encoutered, stopping by user request.")
+                    logging.debug("Error encountered, stopping by user request.")
                     common.force_exit(1)
                 failed_builds.append((appid, build.versionCode))
                 common.deploy_build_log_with_rsync(
