@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+"""Check for updates to applications."""
 #
 # checkupdates.py - part of the FDroid server tools
 # Copyright (C) 2010-2015, Ciaran Gultnieks, ciaran@ciarang.com
@@ -17,6 +18,8 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import configparser
+import git
 import os
 import re
 import urllib.request
@@ -26,12 +29,11 @@ import subprocess
 import sys
 from argparse import ArgumentParser
 import traceback
-import html
-from distutils.version import LooseVersion
 import logging
 import copy
 import urllib.parse
 from pathlib import Path
+from typing import Optional
 
 from . import _
 from . import common
@@ -40,11 +42,36 @@ from . import net
 from .exception import VCSException, NoSubmodulesException, FDroidException, MetaDataException
 
 
-# Check for a new version by looking at a document retrieved via HTTP.
-# The app's Update Check Data field is used to provide the information
-# required.
-def check_http(app):
+# https://gitlab.com/fdroid/checkupdates-runner/-/blob/1861899262a62a4ed08fa24e5449c0368dfb7617/.gitlab-ci.yml#L36
+BOT_EMAIL = 'fdroidci@bubu1.eu'
 
+
+def check_http(app: metadata.App) -> tuple[Optional[str], Optional[int]]:
+    """Check for a new version by looking at a document retrieved via HTTP.
+
+    The app's UpdateCheckData field is used to provide the information
+    required.
+
+    Parameters
+    ----------
+    app
+        The App instance to check for updates for.
+
+    Returns
+    -------
+    version
+        The found versionName or None if the versionName should be ignored
+        according to UpdateCheckIgnore.
+    vercode
+        The found versionCode or None if the versionCode should be ignored
+        according to UpdateCheckIgnore.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.FDroidException`
+        If UpdateCheckData is missing or is an invalid URL or if there is no
+        match for the provided versionName or versionCode regex.
+    """
     if not app.UpdateCheckData:
         raise FDroidException('Missing Update Check Data')
 
@@ -85,12 +112,37 @@ def check_http(app):
     return (version, vercode)
 
 
-def check_tags(app, pattern):
+def check_tags(app: metadata.App, pattern: str) -> tuple[str, int, str]:
     """Check for a new version by looking at the tags in the source repo.
 
     Whether this can be used reliably or not depends on
     the development procedures used by the project's developers. Use it with
     caution, because it's inappropriate for many projects.
+
+    Parameters
+    ----------
+    app
+        The App instance to check for updates for.
+    pattern
+        The pattern a tag needs to match to be considered.
+
+    Returns
+    -------
+    versionName
+        The highest found versionName.
+    versionCode
+        The highest found versionCode.
+    ref
+        The Git reference, commit hash or tag name, of the highest found
+        versionName, versionCode.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.MetaDataException`
+        If this function is not suitable for the RepoType of the app or
+        information is missing to perform this type of check.
+    :exc:`~fdroidserver.exception.FDroidException`
+        If no matching tags or no information whatsoever could be found.
     """
     if app.RepoType == 'srclib':
         build_dir = Path('build/srclib') / app.Repo
@@ -110,7 +162,7 @@ def check_tags(app, pattern):
 
     vcs.gotorevision(None)
 
-    last_build = app.get_last_build()
+    last_build = get_last_build_from_app(app)
 
     try_init_submodules(app, last_build, vcs)
 
@@ -159,6 +211,8 @@ def check_tags(app, pattern):
             if codeex:
                 m = re.search(codeex, filecontent)
                 if not m:
+                    logging.debug(f"UpdateCheckData regex {codeex} for version code"
+                                  f" has no match in tag {tag}")
                     continue
 
                 vercode = m.group(1).strip()
@@ -176,8 +230,12 @@ def check_tags(app, pattern):
             version = tag
             if verex:
                 m = re.search(verex, filecontent)
-                if m:
-                    version = m.group(1)
+                if not m:
+                    logging.debug(f"UpdateCheckData regex {verex} for version name"
+                                  f" has no match in tag {tag}")
+                    continue
+
+                version = m.group(1)
 
             logging.debug("UpdateCheckData found version {0} ({1})"
                           .format(version, vercode))
@@ -218,12 +276,31 @@ def check_tags(app, pattern):
     raise FDroidException(_("Couldn't find any version information"))
 
 
-def check_repomanifest(app, branch=None):
+def check_repomanifest(app: metadata.App, branch: Optional[str] = None) -> tuple[str, int]:
     """Check for a new version by looking at the AndroidManifest.xml at the HEAD of the source repo.
 
     Whether this can be used reliably or not depends on
     the development procedures used by the project's developers. Use it with
     caution, because it's inappropriate for many projects.
+
+    Parameters
+    ----------
+    app
+        The App instance to check for updates for.
+    branch
+        The VCS branch where to search for versionCode, versionName.
+
+    Returns
+    -------
+    versionName
+        The highest found versionName.
+    versionCode
+        The highest found versionCode.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.FDroidException`
+        If no package id or no version information could be found.
     """
     if app.RepoType == 'srclib':
         build_dir = Path('build/srclib') / app.Repo
@@ -246,10 +323,7 @@ def check_repomanifest(app, branch=None):
     elif repotype == 'bzr':
         vcs.gotorevision(None)
 
-    last_build = metadata.Build()
-    if app.get('Builds', []):
-        last_build = app.get('Builds', [])[-1]
-
+    last_build = get_last_build_from_app(app)
     try_init_submodules(app, last_build, vcs)
 
     hpak = None
@@ -274,58 +348,8 @@ def check_repomanifest(app, branch=None):
     raise FDroidException(_("Couldn't find any version information"))
 
 
-def check_repotrunk(app):
-    if app.RepoType == 'srclib':
-        build_dir = Path('build/srclib') / app.Repo
-        repotype = common.getsrclibvcs(app.Repo)
-    else:
-        build_dir = Path('build') / app.id
-        repotype = app.RepoType
-
-    if repotype not in ('git-svn', ):
-        raise MetaDataException(_('RepoTrunk update mode only makes sense in git-svn repositories'))
-
-    # Set up vcs interface and make sure we have the latest code...
-    vcs = common.getvcs(app.RepoType, app.Repo, build_dir)
-
-    vcs.gotorevision(None)
-
-    ref = vcs.getref()
-    return (ref, ref)
-
-
-# Check for a new version by looking at the Google Play Store.
-# Returns (None, "a message") if this didn't work, or (version, None) for
-# the details of the current version.
-def check_gplay(app):
-    time.sleep(15)
-    url = 'https://play.google.com/store/apps/details?id=' + app.id
-    headers = {'User-Agent': 'Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/20100101 Firefox/18.0'}
-    req = urllib.request.Request(url, None, headers)
-    try:
-        resp = urllib.request.urlopen(req, None, 20)  # nosec B310 URL base is hardcoded above
-        page = resp.read().decode()
-    except urllib.error.HTTPError as e:
-        return (None, str(e.code))
-    except Exception as e:
-        return (None, 'Failed:' + str(e))
-
-    version = None
-
-    m = re.search('itemprop="softwareVersion">[ ]*([^<]+)[ ]*</div>', page)
-    if m:
-        version = html.unescape(m.group(1))
-
-    if version == 'Varies with device':
-        return (None, 'Device-variable version, cannot use this method')
-
-    if not version:
-        return (None, "Couldn't find version")
-    return (version.strip(), None)
-
-
-def try_init_submodules(app, last_build, vcs):
-    """Try to init submodules if the last build entry used them.
+def try_init_submodules(app: metadata.App, last_build: metadata.Build, vcs: common.vcs):
+    """Try to init submodules if the last build entry uses them.
 
     They might have been removed from the app's repo in the meantime,
     so if we can't find any submodules we continue with the updates check.
@@ -340,44 +364,87 @@ def try_init_submodules(app, last_build, vcs):
             logging.info("submodule broken for {}".format(_getappname(app)))
 
 
-# Return all directories under startdir that contain any of the manifest
-# files, and thus are probably an Android project.
-def dirs_with_manifest(startdir):
+def dirs_with_manifest(startdir: str):
+    """Find directories containing a manifest file.
+
+    Yield all directories under startdir that contain any of the manifest
+    files, and thus are probably an Android project.
+
+    Parameters
+    ----------
+    startdir
+        Directory to be walked down for search
+
+    Yields
+    ------
+    path : :class:`pathlib.Path` or None
+        A directory that contains a manifest file of an Android project, None if
+        no directory could be found
+    """
     for root, _dirs, files in os.walk(startdir):
         if any(m in files for m in [
                 'AndroidManifest.xml', 'pom.xml', 'build.gradle', 'build.gradle.kts']):
             yield Path(root)
 
 
-# Tries to find a new subdir starting from the root build_dir. Returns said
-# subdir relative to the build dir if found, None otherwise.
-def possible_subdirs(app):
+def possible_subdirs(app: metadata.App):
+    """Try to find a new subdir starting from the root build_dir.
 
+    Yields said subdir relative to the build dir if found, None otherwise.
+
+    Parameters
+    ----------
+    app
+        The app to check for subdirs
+
+    Yields
+    ------
+    subdir : :class:`pathlib.Path` or None
+        A possible subdir, None if no subdir could be found
+    """
     if app.RepoType == 'srclib':
         build_dir = Path('build/srclib') / app.Repo
     else:
         build_dir = Path('build') / app.id
 
-    last_build = app.get_last_build()
+    last_build = get_last_build_from_app(app)
 
     for d in dirs_with_manifest(build_dir):
         m_paths = common.manifest_paths(d, last_build.gradle)
         package = common.parse_androidmanifests(m_paths, app)[2]
-        if package is not None:
+        if package is not None or app.UpdateCheckName == "Ignore":
             subdir = d.relative_to(build_dir)
             logging.debug("Adding possible subdir %s" % subdir)
             yield subdir
 
 
-def _getappname(app):
+def _getappname(app: metadata.App) -> str:
     return common.get_app_display_name(app)
 
 
-def _getcvname(app):
+def _getcvname(app: metadata.App) -> str:
     return '%s (%s)' % (app.CurrentVersion, app.CurrentVersionCode)
 
 
-def fetch_autoname(app, tag):
+def fetch_autoname(app: metadata.App, tag: str) -> Optional[str]:
+    """Fetch AutoName.
+
+    Get the to be displayed name of an app from the source code and adjust the
+    App instance in case it is different name has been found.
+
+    Parameters
+    ----------
+    app
+        The App instance to get the AutoName for.
+    tag
+        Tag to fetch AutoName at.
+
+    Returns
+    -------
+    commitmsg
+        Commit message about the name change.  None in case checking for the
+        name is disabled, a VCSException occured or no name could be found.
+    """
     if not app.RepoType or app.UpdateCheckMode in ('None', 'Static') \
        or app.UpdateCheckName == "Ignore":
         return None
@@ -393,7 +460,7 @@ def fetch_autoname(app, tag):
     except VCSException:
         return None
 
-    last_build = app.get_last_build()
+    last_build = get_last_build_from_app(app)
 
     logging.debug("...fetch auto name from " + str(build_dir))
     new_name = None
@@ -415,7 +482,26 @@ def fetch_autoname(app, tag):
     return commitmsg
 
 
-def operate_vercode(operation, vercode):
+def operate_vercode(operation: str, vercode: int) -> int:
+    """Calculate a new versionCode from a mathematical operation.
+
+    Parameters
+    ----------
+    operation
+        The operation to execute to get the new versionCode.
+    vercode
+        The versionCode for replacing "%c" in the operation.
+
+    Returns
+    -------
+    vercode
+        The new versionCode obtained by executing the operation.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.MetaDataException`
+        If the operation is invalid.
+    """
     if not common.VERCODE_OPERATION_RE.match(operation):
         raise MetaDataException(_('Invalid VercodeOperation: {field}')
                                 .format(field=operation))
@@ -426,9 +512,28 @@ def operate_vercode(operation, vercode):
     return vercode
 
 
-def checkupdates_app(app):
+def checkupdates_app(app: metadata.App, auto: bool, commit: bool = False) -> None:
+    """Check for new versions and updated name of a single app.
+
+    Also write back changes to the metadata file and create a Git commit if
+    requested.
+
+    Parameters
+    ----------
+    app
+        The app to check for updates for.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.MetaDataException`
+        If the app has an invalid UpdateCheckMode or AutoUpdateMode.
+    :exc:`~fdroidserver.exception.FDroidException`
+        If no version information could be found, the current version is newer
+        than the found version, auto-update was requested but an app has no
+        CurrentVersionCode or (Git) commiting the changes failed.
+    """
     # If a change is made, commitmsg should be set to a description of it.
-    # Only if this is set will changes be written back to the metadata.
+    # Only if this is set, changes will be written back to the metadata.
     commitmsg = None
 
     tag = None
@@ -441,8 +546,6 @@ def checkupdates_app(app):
     elif mode.startswith('RepoManifest/'):
         tag = mode[13:]
         (version, vercode) = check_repomanifest(app, tag)
-    elif mode == 'RepoTrunk':
-        (version, vercode) = check_repotrunk(app)
     elif mode == 'HTTP':
         (version, vercode) = check_http(app)
     elif mode in ('None', 'Static'):
@@ -485,7 +588,7 @@ def checkupdates_app(app):
         logging.info('...updating to version %s' % ver)
         commitmsg = 'Update CurrentVersion of %s to %s' % (name, ver)
 
-    if options.auto:
+    if auto:
         mode = app.AutoUpdateMode
         if not app.CurrentVersionCode:
             raise MetaDataException(
@@ -521,19 +624,24 @@ def checkupdates_app(app):
             if not gotcur:
                 newbuilds = copy.deepcopy(builds[-len(vercodes):])
 
-                bullseye_blocklist = [
+                # These are either built-in or invalid in newer system versions
+                bookworm_blocklist = [
                     'apt-get install -y openjdk-11-jdk',
                     'apt-get install openjdk-11-jdk-headless',
                     'apt-get install -y openjdk-11-jdk-headless',
                     'apt-get install -t stretch-backports openjdk-11-jdk-headless openjdk-11-jre-headless',
                     'apt-get install -y -t stretch-backports openjdk-11-jdk-headless openjdk-11-jre-headless',
+                    'apt-get install -y openjdk-17-jdk',
+                    'apt-get install openjdk-17-jdk-headless',
+                    'apt-get install -y openjdk-17-jdk-headless',
                     'update-alternatives --auto java',
+                    'update-java-alternatives -a',
                 ]
 
                 for build in newbuilds:
                     if "sudo" in build:
-                        if any("openjdk-11" in line for line in build["sudo"]):
-                            build["sudo"] = [line for line in build["sudo"] if line not in bullseye_blocklist]
+                        if any("openjdk-11" in line for line in build["sudo"]) or any("openjdk-17" in line for line in build["sudo"]):
+                            build["sudo"] = [line for line in build["sudo"] if line not in bookworm_blocklist]
                         if build["sudo"] == ['apt-get update']:
                             build["sudo"] = ''
 
@@ -563,7 +671,7 @@ def checkupdates_app(app):
 
     if commitmsg:
         metadata.write_metadata(app.metadatapath, app)
-        if options.commit:
+        if commit:
             logging.info("Commiting update for " + app.metadatapath)
             gitcmd = ["git", "commit", "-m", commitmsg]
             if 'auto_author' in config:
@@ -573,7 +681,192 @@ def checkupdates_app(app):
                 raise FDroidException("Git commit failed")
 
 
-def status_update_json(processed, failed):
+def get_last_build_from_app(app: metadata.App) -> metadata.Build:
+    """Get the last build entry of an app."""
+    if app.get('Builds'):
+        return app['Builds'][-1]
+    else:
+        return metadata.Build()
+
+
+def get_upstream_main_branch(git_repo):
+    if len(git_repo.remotes.upstream.refs) == 1:
+        return git_repo.remotes.upstream.refs[0].name
+    for name in ('main', 'master'):
+        if name in git_repo.remotes.upstream.refs:
+            return f'upstream/{name}'
+    try:
+        with git_repo.config_reader() as reader:
+            return 'upstream/%s' % reader.get_value('init', 'defaultBranch')
+    except configparser.NoSectionError:
+        return 'upstream/main'
+
+
+def checkout_appid_branch(appid):
+    """Prepare the working branch named after the appid.
+
+    This sets up everything for checkupdates_app() to run and add
+    commits.  If there is an existing branch named after the appid,
+    and it has commits from users other than the checkupdates-bot,
+    then this will return False.  Otherwise, it returns True.
+
+    The checkupdates-runner must set the committer email address in
+    the git config.  Then any commit with a committer or author that
+    does not match that will be considered to have human edits.  That
+    email address is currently set in:
+    https://gitlab.com/fdroid/checkupdates-runner/-/blob/1861899262a62a4ed08fa24e5449c0368dfb7617/.gitlab-ci.yml#L36
+
+    """
+    logging.debug(f'Creating merge request branch for {appid}')
+    git_repo = git.Repo.init('.')
+    upstream_main = get_upstream_main_branch(git_repo)
+    for remote in git_repo.remotes:
+        remote.fetch()
+    try:
+        git_repo.remotes.origin.fetch(f'{appid}:refs/remotes/origin/{appid}')
+    except Exception as e:
+        logging.warning('"%s" branch not found on origin remote:\n\t%s', appid, e)
+    if appid in git_repo.remotes.origin.refs:
+        start_point = f"origin/{appid}"
+        for commit in git_repo.iter_commits(
+            f'{upstream_main}...{start_point}', right_only=True
+        ):
+            if commit.committer.email != BOT_EMAIL or commit.author.email != BOT_EMAIL:
+                return False
+    else:
+        start_point = upstream_main
+    git_repo.git.checkout('-B', appid, start_point)
+    git_repo.git.rebase(upstream_main, strategy_option='ours', kill_after_timeout=120)
+    return True
+
+
+def get_changes_versus_ref(git_repo, ref, f):
+    changes = []
+    for m in re.findall(
+        r"^[+-].*", git_repo.git.diff(f"{ref}", '--', f), flags=re.MULTILINE
+    ):
+        if not re.match(r"^(\+\+\+|---) ", m):
+            changes.append(m)
+    return changes
+
+
+def push_commits(branch_name='checkupdates', verbose=False):
+    """Make git branch then push commits as merge request.
+
+    The appid is parsed from the actual file that was changed so that
+    only the right branch is ever updated.
+
+    This uses the appid as the standard branch name so that there is
+    only ever one open merge request per-app.  If multiple apps are
+    included in the branch, then 'checkupdates' is used as branch
+    name.  This is to support the old way operating, e.g. in batches.
+
+    This uses GitLab "Push Options" to create a merge request. Git
+    Push Options are config data that can be sent via `git push
+    --push-option=... origin foo`.
+
+    References
+    ----------
+    * https://docs.gitlab.com/ee/user/project/push_options.html
+
+    """
+    git_repo = git.Repo.init('.')
+    upstream_main = get_upstream_main_branch(git_repo)
+    files = set()
+    for commit in git_repo.iter_commits(f'{upstream_main}...HEAD', right_only=True):
+        files.update(commit.stats.files.keys())
+
+    files = list(files)
+    if len(files) == 1:
+        m = re.match(r'metadata/(\S+)\.yml', files[0])
+        if m:
+            branch_name = m.group(1)  # appid
+    if not files:
+        return
+
+    remote = git_repo.remotes.origin
+    if branch_name in remote.refs:
+        if not get_changes_versus_ref(git_repo, f'origin/{branch_name}', files[0]):
+            return
+
+    git_repo.create_head(branch_name, force=True)
+    push_options = [
+        'merge_request.create',
+        'merge_request.remove_source_branch',
+        'merge_request.title=bot: ' + git_repo.branches[branch_name].commit.summary,
+        'merge_request.description='
+        + '~%s checkupdates-bot run %s' % (branch_name, os.getenv('CI_JOB_URL')),
+    ]
+
+    # mark as draft if there are only changes to CurrentVersion:
+    current_version_only = True
+    for m in get_changes_versus_ref(git_repo, upstream_main, files[0]):
+        if not re.match(r"^[-+]CurrentVersion", m):
+            current_version_only = False
+            break
+    if current_version_only:
+        push_options.append('merge_request.draft')
+
+    progress = None
+    if verbose:
+        import clint.textui
+
+        progress_bar = clint.textui.progress.Bar()
+
+        class MyProgressPrinter(git.RemoteProgress):
+            def update(self, op_code, current, maximum=None, message=None):
+                if isinstance(maximum, float):
+                    progress_bar.show(current, maximum)
+
+        progress = MyProgressPrinter()
+
+    pushinfos = remote.push(
+        branch_name,
+        progress=progress,
+        force=True,
+        set_upstream=True,
+        push_option=push_options,
+    )
+
+    for pushinfo in pushinfos:
+        if pushinfo.flags & (
+            git.remote.PushInfo.ERROR
+            | git.remote.PushInfo.REJECTED
+            | git.remote.PushInfo.REMOTE_FAILURE
+            | git.remote.PushInfo.REMOTE_REJECTED
+        ):
+            # Show potentially useful messages from git remote
+            if progress:
+                for line in progress.other_lines:
+                    if line.startswith('remote:'):
+                        logging.debug(line)
+            raise FDroidException(
+                f'{remote.url} push failed: {pushinfo.flags} {pushinfo.summary}'
+            )
+        else:
+            logging.debug(remote.url + ': ' + pushinfo.summary)
+
+
+def prune_empty_appid_branches(git_repo=None, main_branch='main'):
+    """Remove empty branches from checkupdates-bot git remote."""
+    if git_repo is None:
+        git_repo = git.Repo.init('.')
+    upstream_main = get_upstream_main_branch(git_repo)
+    main_branch = upstream_main.split('/')[1]
+
+    remote = git_repo.remotes.origin
+    remote.update(prune=True)
+    merged_branches = git_repo.git().branch(remotes=True, merged=upstream_main).split()
+    for remote_branch in merged_branches:
+        if not remote_branch or '/' not in remote_branch:
+            continue
+        if remote_branch.split('/')[1] not in (main_branch, 'HEAD'):
+            for ref in git_repo.remotes.origin.refs:
+                if remote_branch == ref.name:
+                    remote.push(':%s' % ref.remote_head, force=True)  # rm remote branch
+
+
+def status_update_json(processed: list, failed: dict) -> None:
     """Output a JSON file with metadata about this run."""
     logging.debug(_('Outputting JSON'))
     output = common.setup_status_output(start_timestamp)
@@ -585,13 +878,16 @@ def status_update_json(processed, failed):
 
 
 config = None
-options = None
 start_timestamp = time.gmtime()
 
 
 def main():
+    """Check for updates for one or more apps.
 
-    global config, options
+    The behaviour of this function is influenced by the configuration file as
+    well as command line parameters.
+    """
+    global config
 
     # Parse command line...
     parser = ArgumentParser()
@@ -603,15 +899,15 @@ def main():
                         help=_("Only process apps with auto-updates"))
     parser.add_argument("--commit", action="store_true", default=False,
                         help=_("Commit changes"))
+    parser.add_argument("--merge-request", action="store_true", default=False,
+                        help=_("Commit changes, push, then make a merge request"))
     parser.add_argument("--allow-dirty", action="store_true", default=False,
                         help=_("Run on git repo that has uncommitted changes"))
-    parser.add_argument("--gplay", action="store_true", default=False,
-                        help=_("Only print differences with the Play Store"))
     metadata.add_metadata_arguments(parser)
-    options = parser.parse_args()
+    options = common.parse_args(parser)
     metadata.warnings_action = options.W
 
-    config = common.read_config(options)
+    config = common.read_config()
 
     if not options.allow_dirty:
         status = subprocess.check_output(['git', 'status', '--porcelain'])
@@ -619,35 +915,11 @@ def main():
             logging.error(_('Build metadata git repo has uncommited changes!'))
             sys.exit(1)
 
-    # Get all apps...
-    allapps = metadata.read_metadata()
+    if options.merge_request and not (options.appid and len(options.appid) == 1):
+        logging.error(_('--merge-request only runs on a single appid!'))
+        sys.exit(1)
 
-    apps = common.read_app_args(options.appid, allapps, False)
-
-    if options.gplay:
-        for appid, app in apps.items():
-            version, reason = check_gplay(app)
-            if version is None:
-                if reason == '404':
-                    logging.info("{0} is not in the Play Store".format(_getappname(app)))
-                else:
-                    logging.info("{0} encountered a problem: {1}".format(_getappname(app), reason))
-            if version is not None:
-                stored = app.CurrentVersion
-                if not stored:
-                    logging.info("{0} has no Current Version but has version {1} on the Play Store"
-                                 .format(_getappname(app), version))
-                elif LooseVersion(stored) < LooseVersion(version):
-                    logging.info("{0} has version {1} on the Play Store, which is bigger than {2}"
-                                 .format(_getappname(app), version, stored))
-                else:
-                    if stored != version:
-                        logging.info("{0} has version {1} on the Play Store, which differs from {2}"
-                                     .format(_getappname(app), version, stored))
-                    else:
-                        logging.info("{0} has the same version {1} on the Play Store"
-                                     .format(_getappname(app), version))
-        return
+    apps = common.read_app_args(options.appid)
 
     processed = []
     failed = dict()
@@ -662,7 +934,17 @@ def main():
         logging.info(msg)
 
         try:
-            checkupdates_app(app)
+            if options.merge_request:
+                if not checkout_appid_branch(appid):
+                    msg = _("...checkupdate failed for {appid} : {error}").format(
+                        appid=appid,
+                        error='Open merge request with human edits, skipped.',
+                    )
+                    logging.warning(msg)
+                    failed[appid] = msg
+                    continue
+
+            checkupdates_app(app, options.auto, options.commit or options.merge_request)
             processed.append(appid)
         except Exception as e:
             msg = _("...checkupdate failed for {appid} : {error}").format(appid=appid, error=e)
@@ -671,6 +953,10 @@ def main():
             failed[appid] = str(e)
             exit_code = 1
 
+    if options.appid and options.merge_request:
+        push_commits(verbose=options.verbose)
+        prune_empty_appid_branches()
+
     status_update_json(processed, failed)
     sys.exit(exit_code)
 
