@@ -26,10 +26,11 @@ import json
 import logging
 import os
 import re
+import ruamel.yaml
 import shutil
+import sys
 import tempfile
 import urllib.parse
-import yaml
 import zipfile
 import calendar
 import qrcode
@@ -43,7 +44,7 @@ from . import common
 from . import metadata
 from . import net
 from . import signindex
-from fdroidserver.common import FDroidPopen, FDroidPopenBytes, load_stats_fdroid_signing_key_fingerprints
+from fdroidserver.common import ANTIFEATURES_CONFIG_NAME, CATEGORIES_CONFIG_NAME, CONFIG_CONFIG_NAME, MIRRORS_CONFIG_NAME, RELEASECHANNELS_CONFIG_NAME, DEFAULT_LOCALE, FDroidPopen, FDroidPopenBytes, load_stats_fdroid_signing_key_fingerprints
 from fdroidserver.exception import FDroidException, VerificationException
 
 
@@ -77,7 +78,7 @@ def make(apps, apks, repodir, archive):
         sortedapps[appid] = apps[appid]
 
     repodict = collections.OrderedDict()
-    repodict['timestamp'] = datetime.utcnow().replace(tzinfo=timezone.utc)
+    repodict['timestamp'] = datetime.now(timezone.utc)
     repodict['version'] = METADATA_VERSION
 
     if common.config['repo_maxage'] != 0:
@@ -91,7 +92,7 @@ def make(apps, apks, repodir, archive):
         repodict['address'] = archive_url
         if 'archive_web_base_url' in common.config:
             repodict["webBaseUrl"] = common.config['archive_web_base_url']
-        urlbasepath = os.path.basename(urllib.parse.urlparse(archive_url).path)
+        repo_section = os.path.basename(urllib.parse.urlparse(archive_url).path)
     else:
         repodict['name'] = common.config['repo_name']
         repodict['icon'] = common.config.get('repo_icon', common.default_config['repo_icon'])
@@ -99,27 +100,9 @@ def make(apps, apks, repodir, archive):
         if 'repo_web_base_url' in common.config:
             repodict["webBaseUrl"] = common.config['repo_web_base_url']
         repodict['description'] = common.config['repo_description']
-        urlbasepath = os.path.basename(urllib.parse.urlparse(common.config['repo_url']).path)
+        repo_section = os.path.basename(urllib.parse.urlparse(common.config['repo_url']).path)
 
-    mirrorcheckfailed = False
-    mirrors = []
-    for mirror in common.config.get('mirrors', []):
-        base = os.path.basename(urllib.parse.urlparse(mirror).path.rstrip('/'))
-        if common.config.get('nonstandardwebroot') is not True and base != 'fdroid':
-            logging.error(_("mirror '%s' does not end with 'fdroid'!") % mirror)
-            mirrorcheckfailed = True
-        # must end with / or urljoin strips a whole path segment
-        if mirror.endswith('/'):
-            mirrors.append(urllib.parse.urljoin(mirror, urlbasepath))
-        else:
-            mirrors.append(urllib.parse.urljoin(mirror + '/', urlbasepath))
-    for mirror in common.config.get('servergitmirrors', []):
-        for url in get_mirror_service_urls(mirror):
-            mirrors.append(url + '/' + repodir)
-    if mirrorcheckfailed:
-        raise FDroidException(_("Malformed repository mirrors."))
-    if mirrors:
-        repodict['mirrors'] = mirrors
+    add_mirrors_to_repodict(repo_section, repodict)
 
     requestsdict = collections.OrderedDict()
     for command in ('install', 'uninstall'):
@@ -143,6 +126,13 @@ def make(apps, apks, repodir, archive):
     make_v2(sortedapps, apks, repodir, repodict, requestsdict,
             fdroid_signing_key_fingerprints, archive)
     make_website(sortedapps, repodir, repodict)
+    make_altstore(
+        sortedapps,
+        apks,
+        common.config,
+        repodir,
+        pretty=common.options.pretty,
+    )
 
 
 def _should_file_be_generated(path, magic_string):
@@ -166,12 +156,11 @@ def make_website(apps, repodir, repodict):
     if not os.path.exists(repodir):
         os.makedirs(repodir)
 
-    qrcode.make(link_fingerprinted).save(os.path.join(repodir, "index.png"))
-
     html_name = 'index.html'
     html_file = os.path.join(repodir, html_name)
 
     if _should_file_be_generated(html_file, autogenerate_comment):
+        qrcode.make(link_fingerprinted).save(os.path.join(repodir, "index.png"))
         with open(html_file, 'w') as f:
             name = repodict["name"]
             description = repodict["description"]
@@ -213,10 +202,10 @@ def make_website(apps, repodir, repodict):
         <kbd>
           {number_of_apps}
         </kbd>
-        apps. To add it to your F-Droid client, scan the QR code (click it to enlarge) or use this URL:
+        apps. To add it to your F-Droid app, scan the QR code (click it to enlarge) or use this link:
       </p>
       <p class="center" style="margin-top:.5em">
-        <a href="{link_fingerprinted}">
+        <a href="https://fdroid.link/#{link_fingerprinted}">
           <code style="color:#000000;font-weight:bold;">
             {link}
           </code>
@@ -490,37 +479,6 @@ def dict_diff(source, target):
     return result
 
 
-def file_entry(filename, hash_value=None):
-    meta = {}
-    meta["name"] = "/" + filename.split("/", 1)[1]
-    meta["sha256"] = hash_value or common.sha256sum(filename)
-    meta["size"] = os.stat(filename).st_size
-    return meta
-
-
-def load_locale(name, repodir):
-    lst = {}
-    for yml in Path().glob("config/**/{name}.yml".format(name=name)):
-        locale = yml.parts[1]
-        if len(yml.parts) == 2:
-            locale = "en-US"
-        with open(yml, encoding="utf-8") as fp:
-            elem = yaml.safe_load(fp)
-            for akey, avalue in elem.items():
-                if akey not in lst:
-                    lst[akey] = {}
-                for key, value in avalue.items():
-                    if key not in lst[akey]:
-                        lst[akey][key] = {}
-                    if key == "icon":
-                        shutil.copy(os.path.join("config", value), os.path.join(repodir, "icons"))
-                        lst[akey][key][locale] = file_entry(os.path.join(repodir, "icons", value))
-                    else:
-                        lst[akey][key][locale] = value
-
-    return lst
-
-
 def convert_datetime(obj):
     if isinstance(obj, datetime):
         # Java prefers milliseconds
@@ -551,7 +509,6 @@ def package_metadata(app, repodir):
         "AuthorPhone",
         "AuthorWebSite",
         "Bitcoin",
-        "FlattrID",
         "Liberapay",
         "Litecoin",
         "OpenCollective",
@@ -568,14 +525,14 @@ def package_metadata(app, repodir):
     ):
         element_new = element[:1].lower() + element[1:]
         if element in app and app[element]:
-            meta[element_new] = {"en-US": convert_datetime(app[element])}
+            meta[element_new] = {DEFAULT_LOCALE: convert_datetime(app[element])}
         elif "localized" in app:
             localized = {k: v[element_new] for k, v in app["localized"].items() if element_new in v}
             if localized:
                 meta[element_new] = localized
 
     if "name" not in meta and app["AutoName"]:
-        meta["name"] = {"en-US": app["AutoName"]}
+        meta["name"] = {DEFAULT_LOCALE: app["AutoName"]}
 
     # fdroidserver/metadata.py App default
     if meta["license"] == "Unknown":
@@ -586,7 +543,8 @@ def package_metadata(app, repodir):
 
     # TODO handle different resolutions
     if app.get("icon"):
-        meta["icon"] = {"en-US": file_entry(os.path.join(repodir, "icons", app["icon"]))}
+        icon_path = os.path.join(repodir, "icons", app["icon"])
+        meta["icon"] = {DEFAULT_LOCALE: common.file_entry(icon_path)}
 
     if "iconv2" in app:
         meta["icon"] = app["iconv2"]
@@ -595,6 +553,13 @@ def package_metadata(app, repodir):
 
 
 def convert_version(version, app, repodir):
+    """Convert the internal representation of Builds: into index-v2 versions.
+
+    The diff algorithm of index-v2 uses null/None to mean a field to
+    be removed, so this function handles any Nones that are in the
+    metadata file.
+
+    """
     ver = {}
     if "added" in version:
         ver["added"] = convert_datetime(version["added"])
@@ -604,7 +569,7 @@ def convert_version(version, app, repodir):
     ver["file"] = {
         "name": "/{}".format(version["apkName"]),
         version["hashType"]: version["hash"],
-        "size": version["size"]
+        "size": version["size"],
     }
 
     ipfsCIDv1 = version.get("ipfsCIDv1")
@@ -612,16 +577,16 @@ def convert_version(version, app, repodir):
         ver["file"]["ipfsCIDv1"] = ipfsCIDv1
 
     if "srcname" in version:
-        ver["src"] = file_entry(os.path.join(repodir, version["srcname"]))
+        ver["src"] = common.file_entry(os.path.join(repodir, version["srcname"]))
 
     if "obbMainFile" in version:
-        ver["obbMainFile"] = file_entry(
+        ver["obbMainFile"] = common.file_entry(
             os.path.join(repodir, version["obbMainFile"]),
             version["obbMainFileSha256"],
         )
 
     if "obbPatchFile" in version:
-        ver["obbPatchFile"] = file_entry(
+        ver["obbPatchFile"] = common.file_entry(
             os.path.join(repodir, version["obbPatchFile"]),
             version["obbPatchFileSha256"],
         )
@@ -668,26 +633,28 @@ def convert_version(version, app, repodir):
                 else:
                     manifest[en].append({"name": perm[0]})
 
-    if "AntiFeatures" in app and app["AntiFeatures"]:
-        ver["antiFeatures"] = {}
-        for antif in app["AntiFeatures"]:
-            # TODO: get reasons from fdroiddata
-            # ver["antiFeatures"][antif] = {"en-US": "reason"}
-            ver["antiFeatures"][antif] = {}
-
-    if "AntiFeatures" in version and version["AntiFeatures"]:
-        if "antiFeatures" not in ver:
-            ver["antiFeatures"] = {}
-        for antif in version["AntiFeatures"]:
-            # TODO: get reasons from fdroiddata
-            # ver["antiFeatures"][antif] = {"en-US": "reason"}
-            ver["antiFeatures"][antif] = {}
+    # index-v2 has only per-version antifeatures, not per package.
+    antiFeatures = app.get('AntiFeatures', {}).copy()
+    for name, descdict in version.get('antiFeatures', dict()).items():
+        antiFeatures[name] = descdict
+    if antiFeatures:
+        ver['antiFeatures'] = {
+            k: dict(sorted(antiFeatures[k].items())) for k in sorted(antiFeatures)
+        }
 
     if "versionCode" in version:
         if version["versionCode"] > app["CurrentVersionCode"]:
-            ver["releaseChannels"] = ["Beta"]
+            ver[RELEASECHANNELS_CONFIG_NAME] = ["Beta"]
 
-    for build in app.get('Builds', []):
+    builds = app.get("Builds", [])
+
+    if len(builds) > 0 and version["versionCode"] == builds[-1]["versionCode"]:
+        if "localized" in app:
+            localized = {k: v["whatsNew"] for k, v in app["localized"].items() if "whatsNew" in v}
+            if localized:
+                ver["whatsNew"] = localized
+
+    for build in builds:
         if build['versionCode'] == version['versionCode'] and "whatsNew" in build:
             ver["whatsNew"] = build["whatsNew"]
             break
@@ -698,40 +665,37 @@ def convert_version(version, app, repodir):
 def v2_repo(repodict, repodir, archive):
     repo = {}
 
-    repo["name"] = {"en-US": repodict["name"]}
-    repo["description"] = {"en-US": repodict["description"]}
-    repo["icon"] = {"en-US": file_entry("{}/icons/{}".format(repodir, repodict["icon"]))}
+    repo["name"] = {DEFAULT_LOCALE: repodict["name"]}
+    repo["description"] = {DEFAULT_LOCALE: repodict["description"]}
+    repo["icon"] = {
+        DEFAULT_LOCALE: common.file_entry("%s/icons/%s" % (repodir, repodict["icon"]))
+    }
 
-    config = load_locale("config", repodir)
+    config = common.load_localized_config(CONFIG_CONFIG_NAME, repodir)
     if config:
         repo["name"] = config["archive" if archive else "repo"]["name"]
         repo["description"] = config["archive" if archive else "repo"]["description"]
         repo["icon"] = config["archive" if archive else "repo"]["icon"]
 
     repo["address"] = repodict["address"]
+    if "mirrors" in repodict:
+        repo["mirrors"] = repodict["mirrors"]
     if "webBaseUrl" in repodict:
         repo["webBaseUrl"] = repodict["webBaseUrl"]
 
-    if "mirrors" in repodict:
-        repo["mirrors"] = [{"url": mirror} for mirror in repodict["mirrors"]]
-
-        # the first entry is traditionally the primary mirror
-        if repodict['address'] not in repodict["mirrors"]:
-            repo["mirrors"].insert(0, {"url": repodict['address'], "isPrimary": True})
-
     repo["timestamp"] = repodict["timestamp"]
 
-    antiFeatures = load_locale("antiFeatures", repodir)
+    antiFeatures = common.load_localized_config(ANTIFEATURES_CONFIG_NAME, repodir)
     if antiFeatures:
-        repo["antiFeatures"] = antiFeatures
+        repo[ANTIFEATURES_CONFIG_NAME] = antiFeatures
 
-    categories = load_locale("categories", repodir)
+    categories = common.load_localized_config(CATEGORIES_CONFIG_NAME, repodir)
     if categories:
-        repo["categories"] = categories
+        repo[CATEGORIES_CONFIG_NAME] = categories
 
-    channels = load_locale("channels", repodir)
-    if channels:
-        repo["releaseChannels"] = channels
+    releaseChannels = common.load_localized_config(RELEASECHANNELS_CONFIG_NAME, repodir)
+    if releaseChannels:
+        repo[RELEASECHANNELS_CONFIG_NAME] = releaseChannels
 
     return repo
 
@@ -754,7 +718,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
 
     output = collections.OrderedDict()
     output["repo"] = v2_repo(repodict, repodir, archive)
-    if requestsdict and requestsdict["install"] or requestsdict["uninstall"]:
+    if requestsdict and (requestsdict["install"] or requestsdict["uninstall"]):
         output["repo"]["requests"] = requestsdict
 
     # establish sort order of the index
@@ -762,6 +726,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
 
     output_packages = collections.OrderedDict()
     output['packages'] = output_packages
+    categories_used_by_apps = set()
     for package in packages:
         packageName = package['packageName']
         if packageName not in apps:
@@ -781,7 +746,9 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         else:
             packagelist = {}
             output_packages[packageName] = packagelist
-            packagelist["metadata"] = package_metadata(apps[packageName], repodir)
+            app = apps[packageName]
+            categories_used_by_apps.update(app.get('Categories', []))
+            packagelist["metadata"] = package_metadata(app, repodir)
             if "signer" in package:
                 packagelist["metadata"]["preferredSigner"] = package["signer"]
 
@@ -789,6 +756,19 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
 
         packagelist["versions"][package["hash"]] = convert_version(package, apps[packageName], repodir)
 
+    if categories_used_by_apps and not output['repo'].get(CATEGORIES_CONFIG_NAME):
+        output['repo'][CATEGORIES_CONFIG_NAME] = dict()
+    # include definitions for "auto-defined" categories, e.g. just used in app metadata
+    for category in sorted(categories_used_by_apps):
+        if category not in output['repo'][CATEGORIES_CONFIG_NAME]:
+            output['repo'][CATEGORIES_CONFIG_NAME][category] = {"name": {DEFAULT_LOCALE: category}}
+    # do not include defined categories if no apps use them
+    for category in list(output['repo'].get(CATEGORIES_CONFIG_NAME, list())):
+        if category not in categories_used_by_apps:
+            del output['repo'][CATEGORIES_CONFIG_NAME][category]
+            msg = _('Category "{category}" defined but not used for any apps!')
+            logging.warning(msg.format(category=category))
+
     entry = {}
     entry["timestamp"] = repodict["timestamp"]
 
@@ -811,7 +791,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         else:
             json.dump(output, fp, default=_index_encoder_default, ensure_ascii=False)
 
-    entry["index"] = file_entry(index_file)
+    entry["index"] = common.file_entry(index_file)
     entry["index"]["numPackages"] = len(output.get("packages", []))
 
     indexes = sorted(Path().glob("tmp/{}*.json".format(repodir)), key=lambda x: x.name)
@@ -838,7 +818,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
             else:
                 json.dump(diff, fp, default=_index_encoder_default, ensure_ascii=False)
 
-        entry["diffs"][old["repo"]["timestamp"]] = file_entry(diff_file)
+        entry["diffs"][old["repo"]["timestamp"]] = common.file_entry(diff_file)
         entry["diffs"][old["repo"]["timestamp"]]["numPackages"] = len(diff.get("packages", []))
 
     json_name = "entry.json"
@@ -874,21 +854,30 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         raise TypeError(repr(obj) + " is not JSON serializable")
 
     output = collections.OrderedDict()
-    output['repo'] = repodict
+    output['repo'] = repodict.copy()
     output['requests'] = requestsdict
 
+    # index-v1 only supports a list of URL strings for additional mirrors
+    mirrors = []
+    for mirror in repodict.get('mirrors', []):
+        url = mirror['url']
+        if url != repodict['address']:
+            mirrors.append(mirror['url'])
+    if mirrors:
+        output['repo']['mirrors'] = mirrors
+
     # establish sort order of the index
     v1_sort_packages(packages, fdroid_signing_key_fingerprints)
 
     appslist = []
     output['apps'] = appslist
-    for packageName, appdict in apps.items():
+    for packageName, app_dict in apps.items():
         d = collections.OrderedDict()
         appslist.append(d)
-        for k, v in sorted(appdict.items()):
+        for k, v in sorted(app_dict.items()):
             if not v:
                 continue
-            if k in ('Builds', 'comments', 'metadatapath',
+            if k in ('Builds', 'metadatapath',
                      'ArchivePolicy', 'AutoName', 'AutoUpdateMode', 'MaintainerNotes',
                      'Provides', 'Repo', 'RepoType', 'RequiresRoot',
                      'UpdateCheckData', 'UpdateCheckIgnore', 'UpdateCheckMode',
@@ -911,18 +900,19 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
             d[k] = v
 
     # establish sort order in lists, sets, and localized dicts
-    for app in output['apps']:
-        localized = app.get('localized')
+    for app_dict in output['apps']:
+        localized = app_dict.get('localized')
         if localized:
             lordered = collections.OrderedDict()
             for lkey, lvalue in sorted(localized.items()):
                 lordered[lkey] = collections.OrderedDict()
                 for ikey, iname in sorted(lvalue.items()):
                     lordered[lkey][ikey] = iname
-            app['localized'] = lordered
-        antiFeatures = app.get('AntiFeatures')
+            app_dict['localized'] = lordered
+        # v1 uses a list of keys for Anti-Features
+        antiFeatures = app_dict.get('antiFeatures', dict()).keys()
         if antiFeatures:
-            app['AntiFeatures'] = sorted(set(antiFeatures))
+            app_dict['antiFeatures'] = sorted(set(antiFeatures))
 
     output_packages = collections.OrderedDict()
     output['packages'] = output_packages
@@ -952,6 +942,9 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
                 continue
             if k in ('icon', 'icons', 'icons_src', 'ipfsCIDv1', 'name'):
                 continue
+            if k == 'antiFeatures':
+                d[k] = sorted(v.keys())
+                continue
             d[k] = v
 
     json_name = 'index-v1.json'
@@ -1058,7 +1051,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         lkey = key[:1].lower() + key[1:]
         localized = app.get('localized')
         if not value and localized:
-            for lang in ['en-US'] + [x for x in localized.keys()]:
+            for lang in [DEFAULT_LOCALE] + [x for x in localized.keys()]:
                 if not lang.startswith('en'):
                     continue
                 if lang in localized:
@@ -1090,8 +1083,11 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
     repoel.setAttribute("version", str(repodict['version']))
 
     addElement('description', repodict['description'], doc, repoel)
+    # index v0 only supports a list of URL strings for additional mirrors
     for mirror in repodict.get('mirrors', []):
-        addElement('mirror', mirror, doc, repoel)
+        url = mirror['url']
+        if url != repodict['address']:
+            addElement('mirror', url, doc, repoel)
 
     root.appendChild(repoel)
 
@@ -1101,8 +1097,8 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
             root.appendChild(element)
             element.setAttribute('packageName', packageName)
 
-    for appid, appdict in apps.items():
-        app = metadata.App(appdict)
+    for appid, app_dict in apps.items():
+        app = metadata.App(app_dict)
 
         if app.get('Disabled') is not None:
             continue
@@ -1172,7 +1168,6 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         addElementNonEmpty('donate', app.Donate, doc, apel)
         addElementNonEmpty('bitcoin', app.Bitcoin, doc, apel)
         addElementNonEmpty('litecoin', app.Litecoin, doc, apel)
-        addElementNonEmpty('flattr', app.FlattrID, doc, apel)
         addElementNonEmpty('openCollective', app.OpenCollective, doc, apel)
 
         # These elements actually refer to the current version (i.e. which
@@ -1191,10 +1186,11 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
         # doesn't have to do any work by default...
         apklist = sorted(apklist, key=lambda apk: apk['versionCode'], reverse=True)
 
+        antiFeatures = list(app.AntiFeatures)
         if 'antiFeatures' in apklist[0]:
-            app.AntiFeatures.extend(apklist[0]['antiFeatures'])
-        if app.AntiFeatures:
-            afout = sorted(set(app.AntiFeatures))
+            antiFeatures.extend(apklist[0]['antiFeatures'])
+        if antiFeatures:
+            afout = sorted(set(antiFeatures))
             addElementNonEmpty('antifeatures', ','.join(afout), doc, apel)
 
         # Check for duplicates - they will make the client unhappy...
@@ -1296,7 +1292,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
             namefield = common.config['current_version_name_source']
             name = app.get(namefield)
             if not name and namefield == 'Name':
-                name = app.get('localized', {}).get('en-US', {}).get('name')
+                name = app.get('localized', {}).get(DEFAULT_LOCALE, {}).get('name')
             if not name:
                 name = app.id
             sanitized_name = re.sub(b'''[ '"&%?+=/]''', b'', str(name).encode('utf-8'))
@@ -1314,6 +1310,29 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
                         os.remove(siglinkname)
                     os.symlink(sigfile_path, siglinkname)
 
+    if sys.version_info.minor >= 13:
+        # Python 3.13 changed minidom so it no longer converts " to an XML entity.
+        # https://github.com/python/cpython/commit/154477be722ae5c4e18d22d0860e284006b09c4f
+        # This just puts back the previous implementation, with black code format.
+        import inspect
+        import xml.dom.minidom
+
+        def _write_data(writer, text, attr):  # pylint: disable=unused-argument
+            if text:
+                text = (
+                    text.replace('&', '&amp;')
+                    .replace('<', '&lt;')
+                    .replace('"', '&quot;')
+                    .replace('>', '&gt;')
+                )
+            writer.write(text)
+
+        argnames = tuple(inspect.signature(xml.dom.minidom._write_data).parameters)
+        if argnames == ('writer', 'text', 'attr'):
+            xml.dom.minidom._write_data = _write_data
+        else:
+            logging.warning('Failed to monkey patch minidom for index.xml support!')
+
     if common.options.pretty:
         output = doc.toprettyxml(encoding='utf-8')
     else:
@@ -1398,7 +1417,104 @@ def extract_pubkey():
     return hexlify(pubkey), repo_pubkey_fingerprint
 
 
-def get_mirror_service_urls(url):
+def add_mirrors_to_repodict(repo_section, repodict):
+    """Convert config into final dict of mirror metadata for the repo.
+
+    Internally and in index-v2, mirrors is a list of dicts, but it can
+    be specified in the config as a string or list of strings.  Also,
+    index v0 and v1 use a list of URL strings as the data structure.
+
+    The first entry is traditionally the primary mirror and canonical
+    URL.  'mirrors' should not be present in the index if there is
+    only the canonical URL, and no other mirrors.
+
+    The metadata items for each mirror entry are sorted by key to
+    ensure minimum diffs in the index files.
+
+    """
+    mirrors_config = common.config.get('mirrors', [])
+    if type(mirrors_config) not in (list, tuple):
+        mirrors_config = [mirrors_config]
+
+    mirrors_yml = Path(f'config/{MIRRORS_CONFIG_NAME}.yml')
+    if mirrors_yml.exists():
+        if mirrors_config:
+            raise FDroidException(
+                _('mirrors set twice, in config.yml and {path}!').format(
+                    path=mirrors_yml
+                )
+            )
+        with mirrors_yml.open() as fp:
+            mirrors_config = ruamel.yaml.YAML(typ='safe').load(fp)
+        if not isinstance(mirrors_config, list):
+            msg = _('{path} is not list, but a {datatype}!')
+            raise TypeError(
+                msg.format(path=mirrors_yml, datatype=type(mirrors_config).__name__)
+            )
+
+    if type(mirrors_config) not in (list, tuple, set):
+        msg = 'In config.yml, mirrors: is not list, but a {datatype}!'
+        raise TypeError(msg.format(datatype=type(mirrors_config).__name__))
+
+    mirrorcheckfailed = False
+    mirrors = []
+    urls = set()
+    for mirror in mirrors_config:
+        if isinstance(mirror, str):
+            mirror = {'url': mirror}
+        elif not isinstance(mirror, dict):
+            logging.error(
+                _('Bad entry type "{mirrortype}" in mirrors config: {mirror}').format(
+                    mirrortype=type(mirror), mirror=mirror
+                )
+            )
+            mirrorcheckfailed = True
+            continue
+        config_url = mirror['url']
+        base = os.path.basename(urllib.parse.urlparse(config_url).path.rstrip('/'))
+        if common.config.get('nonstandardwebroot') is not True and base != 'fdroid':
+            logging.error(_("mirror '%s' does not end with 'fdroid'!") % config_url)
+            mirrorcheckfailed = True
+        # must end with / or urljoin strips a whole path segment
+        if config_url.endswith('/'):
+            mirror['url'] = urllib.parse.urljoin(config_url, repo_section)
+        else:
+            mirror['url'] = urllib.parse.urljoin(config_url + '/', repo_section)
+        mirrors.append(mirror)
+        if mirror['url'] in urls:
+            mirrorcheckfailed = True
+            logging.error(
+                _('Duplicate entry "%s" in mirrors config!') % mirror['url']
+            )
+        urls.add(mirror['url'])
+    for mirror in common.config.get('servergitmirrors', []):
+        for url in get_mirror_service_urls(mirror):
+            mirrors.append({'url': url + '/' + repo_section})
+    if mirrorcheckfailed:
+        raise FDroidException(_("Malformed repository mirrors."))
+
+    if not mirrors:
+        return
+
+    repodict['mirrors'] = []
+    canonical_url = repodict['address']
+    found_primary = False
+    for mirror in mirrors:
+        if canonical_url == mirror['url']:
+            found_primary = True
+            mirror['isPrimary'] = True
+            sortedmirror = dict()
+            for k in sorted(mirror.keys()):
+                sortedmirror[k] = mirror[k]
+            repodict['mirrors'].insert(0, sortedmirror)
+        else:
+            repodict['mirrors'].append(mirror)
+
+    if repodict['mirrors'] and not found_primary:
+        repodict['mirrors'].insert(0, {'isPrimary': True, 'url': repodict['address']})
+
+
+def get_mirror_service_urls(mirror):
     """Get direct URLs from git service for use by fdroidclient.
 
     Via 'servergitmirrors', fdroidserver can create and push a mirror
@@ -1416,6 +1532,7 @@ def get_mirror_service_urls(url):
     information about the repo available to end user.
 
     """
+    url = mirror['url']
     if url.startswith('git@'):
         url = re.sub(r'^git@([^:]+):(.+)', r'https://\1/\2', url)
 
@@ -1438,7 +1555,10 @@ def get_mirror_service_urls(url):
         urls.append('/'.join(segments))
     elif hostname == "gitlab.com":
         git_mirror_path = os.path.join('git-mirror', folder)
-        if common.get_dir_size(git_mirror_path) <= common.GITLAB_COM_PAGES_MAX_SIZE:
+        if (
+            mirror.get('index_only')
+            or common.get_dir_size(git_mirror_path) <= common.GITLAB_COM_PAGES_MAX_SIZE
+        ):
             # Gitlab-like Pages segments "https://user.gitlab.io/repo/folder"
             gitlab_pages = ["https:", "", user + ".gitlab.io", repo, folder]
             urls.append('/'.join(gitlab_pages))
@@ -1536,7 +1656,7 @@ def download_repo_index_v1(url_str, etag=None, verify_fingerprint=True, timeout=
         return index, new_etag
 
 
-def download_repo_index_v2(url_str, etag=None, verify_fingerprint=True, timeout=600):
+def download_repo_index_v2(url_str, etag=None, verify_fingerprint=True, timeout=None):
     """Download and verifies index v2 file, then returns its data.
 
     Downloads the repository index from the given :param url_str and
@@ -1555,8 +1675,13 @@ def download_repo_index_v2(url_str, etag=None, verify_fingerprint=True, timeout=
         - The new eTag as returned by the HTTP request
 
     """
+    etag  # etag is unused but needs to be there to keep the same API as the earlier functions.
+
     url = urllib.parse.urlsplit(url_str)
 
+    if timeout is not None:
+        logging.warning('"timeout" argument of download_repo_index_v2() is deprecated!')
+
     fingerprint = None
     if verify_fingerprint:
         query = urllib.parse.parse_qs(url.query)
@@ -1568,29 +1693,22 @@ def download_repo_index_v2(url_str, etag=None, verify_fingerprint=True, timeout=
         path = url.path.rsplit('/', 1)[0]
     else:
         path = url.path.rstrip('/')
+    url = urllib.parse.SplitResult(url.scheme, url.netloc, path, '', '')
 
-    url = urllib.parse.SplitResult(url.scheme, url.netloc, path + '/entry.jar', '', '')
-    download, new_etag = net.http_get(url.geturl(), etag, timeout)
+    mirrors = common.get_mirrors(url, 'entry.jar')
+    f = net.download_using_mirrors(mirrors)
+    entry, public_key, fingerprint = get_index_from_jar(f, fingerprint)
 
-    if download is None:
-        return None, new_etag
-
-    # jarsigner is used to verify the JAR, it requires a file for input
-    with tempfile.TemporaryDirectory() as dirname:
-        with (Path(dirname) / 'entry.jar').open('wb') as fp:
-            fp.write(download)
-            fp.flush()
-            entry, public_key, fingerprint = get_index_from_jar(fp.name, fingerprint)
-
-    name = entry['index']['name']
     sha256 = entry['index']['sha256']
-    url = urllib.parse.SplitResult(url.scheme, url.netloc, path + name, '', '')
-    index, _ignored = net.http_get(url.geturl(), None, timeout)
+    mirrors = common.get_mirrors(url, entry['index']['name'][1:])
+    f = net.download_using_mirrors(mirrors)
+    with open(f, 'rb') as fp:
+        index = fp.read()
     if sha256 != hashlib.sha256(index).hexdigest():
         raise VerificationException(
             _("SHA-256 of {url} does not match entry!").format(url=url)
         )
-    return json.loads(index), new_etag
+    return json.loads(index), None
 
 
 def get_index_from_jar(jarfile, fingerprint=None, allow_deprecated=False):
@@ -1661,3 +1779,100 @@ def get_public_key_from_jar(jar):
     public_key_fingerprint = common.get_cert_fingerprint(public_key).replace(' ', '')
 
     return public_key, public_key_fingerprint
+
+
+def make_altstore(apps, apks, config, repodir, pretty=False):
+    """Assemble altstore-index.json for iOS (.ipa) apps.
+
+    builds index files based on:
+    https://faq.altstore.io/distribute-your-apps/make-a-source
+    https://faq.altstore.io/distribute-your-apps/updating-apps
+    """
+    if not any(Path(repodir).glob('*.ipa')):
+        # no IPA files present in repo, nothing to do here, exiting early
+        return
+
+    indent = 2 if pretty else None
+    # for now alt-store support is english only
+    for lang in ['en']:
+
+        # prepare minimal altstore index
+        idx = {
+            'name': config['repo_name'],
+            "apps": [],
+            "news": [],
+        }
+
+        # add optional values if available
+        # idx["subtitle"] F-Droid doesn't have a corresponding value
+        if config.get("repo_description"):
+            idx['description'] = config['repo_description']
+        if (Path(repodir) / 'icons' / config['repo_icon']).exists():
+            idx['iconURL'] = f"{config['repo_url']}/icons/{config['repo_icon']}"
+        # idx["headerURL"] F-Droid doesn't have a corresponding value
+        # idx["website"] F-Droid doesn't have a corresponding value
+        # idx["patreonURL"] F-Droid doesn't have a corresponding value
+        # idx["tintColor"] F-Droid doesn't have a corresponding value
+        # idx["featuredApps"] = [] maybe mappable to F-Droids what's new?
+
+        # assemble "apps"
+        for packageName, app in apps.items():
+            app_name = app.get("Name") or app.get("AutoName")
+            icon_url = "{}{}".format(
+                config['repo_url'],
+                app.get('iconv2', {}).get(DEFAULT_LOCALE, {}).get('name', ''),
+            )
+            screenshot_urls = [
+                "{}{}".format(config["repo_url"], s["name"])
+                for s in app.get("screenshots", {})
+                .get("phone", {})
+                .get(DEFAULT_LOCALE, {})
+            ]
+
+            a = {
+                "name": app_name,
+                'bundleIdentifier': packageName,
+                'developerName': app.get("AuthorName") or f"{app_name} team",
+                'iconURL': icon_url,
+                "localizedDescription": "",
+                'appPermissions': {
+                    "entitlements": set(),
+                    "privacy": {},
+                },
+                'versions': [],
+            }
+
+            if app.get('summary'):
+                a['subtitle'] = app['summary']
+            # a["tintColor"] F-Droid doesn't have a corresponding value
+            # a["category"] F-Droid doesn't have a corresponding value
+            # a['patreon'] F-Droid doesn't have a corresponding value
+            a["screenshots"] = screenshot_urls
+
+            # populate 'versions'
+            for apk in apks:
+                last4 = apk.get('apkName', '').lower()[-4:]
+                if apk['packageName'] == packageName and last4 == '.ipa':
+                    v = {
+                        "version": apk["versionName"],
+                        "date": apk["added"].isoformat(),
+                        "downloadURL": f"{config['repo_url']}/{apk['apkName']}",
+                        "size": apk['size'],
+                    }
+
+                    # v['localizedDescription'] maybe what's new text?
+                    v["minOSVersion"] = apk["ipa_MinimumOSVersion"]
+                    v["maxOSVersion"] = apk["ipa_DTPlatformVersion"]
+
+                    # writing this spot here has the effect that always the
+                    # permissions of the latest processed permissions list used
+                    a['appPermissions']['privacy'] = apk['ipa_permissions']
+                    a['appPermissions']['entitlements'] = list(apk['ipa_entitlements'])
+
+                    a['versions'].append(v)
+
+            if len(a['versions']) > 0:
+                idx['apps'].append(a)
+
+        with open(Path(repodir) / 'altstore-index.json', "w", encoding="utf-8") as f:
+            json.dump(idx, f, indent=indent)
