@@ -32,7 +32,6 @@ from . import common
 from .exception import FDroidException
 
 config = {}
-options = None
 
 
 def disable_in_config(key, value):
@@ -49,8 +48,7 @@ def disable_in_config(key, value):
 
 
 def main():
-
-    global options, config
+    global config
 
     # Parse command line...
     parser = ArgumentParser()
@@ -82,7 +80,9 @@ def main():
         default=False,
         help=_("Do not prompt for Android SDK path, just fail"),
     )
-    options = parser.parse_args()
+    options = common.parse_args(parser)
+
+    common.set_console_logging(options.verbose, options.color)
 
     fdroiddir = os.getcwd()
     test_config = dict()
@@ -115,7 +115,7 @@ def main():
             test_config['sdk_path'] = default_sdk_path
 
         if not common.test_sdk_exists(test_config):
-            del (test_config['sdk_path'])
+            del test_config['sdk_path']
             while not options.no_prompt:
                 try:
                     s = input(
@@ -170,7 +170,7 @@ def main():
         raise FDroidException('Repository already exists.')
 
     # now that we have a local config.yml, read configuration...
-    config = common.read_config(options)
+    config = common.read_config()
 
     # the NDK is optional and there may be multiple versions of it, so it's
     # left for the user to configure
@@ -281,14 +281,19 @@ def main():
     msg += '\n\n'
     msg += (
         _(
-            '''To complete the setup, add your APKs to "%s"
+            """To complete the setup, add your APKs to "%s"
 then run "fdroid update -c; fdroid update".  You might also want to edit
 "config.yml" to set the URL, repo name, and more.  You should also set up
 a signing key (a temporary one might have been automatically generated).
 
 For more info: https://f-droid.org/docs/Setup_an_F-Droid_App_Repo
-and https://f-droid.org/docs/Signing_Process'''
+and https://f-droid.org/docs/Signing_Process"""
         )
         % os.path.join(fdroiddir, 'repo')
     )
-    logging.info(msg)
+    if not options.quiet:
+        # normally, INFO is only shown with --verbose, but show this unless --quiet
+        logger = logging.getLogger()
+        logger.setLevel(logging.INFO)
+        logger.info(msg)
+    logging.shutdown()
