@@ -16,20 +16,19 @@
 # You should have received a copy of the GNU Affero General Public Licen
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from argparse import ArgumentParser
+import difflib
+import platform
 import re
 import sys
-import platform
 import urllib.parse
+from argparse import ArgumentParser
 from pathlib import Path
 
-from . import _
-from . import common
-from . import metadata
-from . import rewritemeta
+import ruamel.yaml
+
+from . import _, common, metadata, rewritemeta
 
 config = None
-options = None
 
 
 def enforce_https(domain):
@@ -139,9 +138,11 @@ http_checks = (
         ),
         (
             re.compile(
-                r'^https://[^/]*(github|gitlab|bitbucket|rawgit|githubusercontent)\.[a-zA-Z]+/([^/]+/){2,3}master/'
+                r'^https://[^/]*(github|gitlab|bitbucket|rawgit|githubusercontent)\.[a-zA-Z]+/([^/]+/){2,3}(master|main)/'
+            ),
+            _(
+                "Use /HEAD instead of /master or /main to point at a file in the default branch"
             ),
-            _("Use /HEAD instead of /master to point at a file in the default branch"),
         ),
     ]
 )
@@ -158,10 +159,6 @@ regex_checks = {
     ],
     'Donate': http_checks
     + [
-        (
-            re.compile(r'.*flattr\.com'),
-            _("Flattr donation methods belong in the FlattrID: field"),
-        ),
         (
             re.compile(r'.*liberapay\.com'),
             _("Liberapay donation methods belong in the Liberapay: field"),
@@ -210,7 +207,7 @@ regex_checks = {
             _("Forbidden HTML tags"),
         ),
         (
-            re.compile(r'''.*\s+src=["']javascript:.*'''),
+            re.compile(r""".*\s+src=["']javascript:.*"""),
             _("Javascript in HTML src attributes"),
         ),
     ],
@@ -220,6 +217,32 @@ locale_pattern = re.compile(r"[a-z]{2,3}(-([A-Z][a-zA-Z]+|\d+|[a-z]+))*")
 
 versioncode_check_pattern = re.compile(r"(\\d|\[(0-9|\\d)_?(a-fA-F)?])[+]")
 
+ANTIFEATURES_KEYS = None
+ANTIFEATURES_PATTERN = None
+CATEGORIES_KEYS = list()
+
+
+def load_antiFeatures_config():
+    """Lazy loading, since it might read a lot of files."""
+    global ANTIFEATURES_KEYS, ANTIFEATURES_PATTERN
+    k = common.ANTIFEATURES_CONFIG_NAME
+    if not ANTIFEATURES_KEYS or k not in common.config:
+        common.config[k] = common.load_localized_config(k, 'repo')
+        ANTIFEATURES_KEYS = sorted(common.config[k].keys())
+        ANTIFEATURES_PATTERN = ','.join(ANTIFEATURES_KEYS)
+
+
+def load_categories_config():
+    """Lazy loading, since it might read a lot of files."""
+    global CATEGORIES_KEYS
+    k = common.CATEGORIES_CONFIG_NAME
+    if not CATEGORIES_KEYS:
+        if config and k in config:
+            CATEGORIES_KEYS = config[k]
+        else:
+            config[k] = common.load_localized_config(k, 'repo')
+            CATEGORIES_KEYS = list(config[k].keys())
+
 
 def check_regexes(app):
     for f, checks in regex_checks.items():
@@ -348,8 +371,7 @@ def check_checkupdates_ran(app):
     if filling_ucms.match(app.UpdateCheckMode):
         if not app.AutoName and not app.CurrentVersion and app.CurrentVersionCode == 0:
             yield _(
-                "UpdateCheckMode is set but it looks like"
-                "checkupdates hasn't been run yet"
+                "UpdateCheckMode is set but it looks like checkupdates hasn't been run yet."
             )
 
 
@@ -358,32 +380,10 @@ def check_empty_fields(app):
         yield _("Categories are not set")
 
 
-all_categories = set(
-    [
-        "Connectivity",
-        "Development",
-        "Games",
-        "Graphics",
-        "Internet",
-        "Money",
-        "Multimedia",
-        "Navigation",
-        "Phone & SMS",
-        "Reading",
-        "Science & Education",
-        "Security",
-        "Sports & Health",
-        "System",
-        "Theming",
-        "Time",
-        "Writing",
-    ]
-)
-
-
 def check_categories(app):
+    """App uses 'Categories' key and parsed config uses 'categories' key."""
     for categ in app.Categories:
-        if categ not in all_categories:
+        if categ not in CATEGORIES_KEYS:
             yield _("Categories '%s' is not valid" % categ)
 
 
@@ -429,27 +429,6 @@ def check_mediawiki_links(app):
                 yield _("URL {url} in Description: {error}").format(url=url, error=r)
 
 
-def check_bulleted_lists(app):
-    validchars = ['*', '#']
-    lchar = ''
-    lcount = 0
-    for line in app.Description.splitlines():
-        if len(line) < 1:
-            lcount = 0
-            continue
-
-        if line[0] == lchar and line[1] == ' ':
-            lcount += 1
-            if lcount > 2 and lchar not in validchars:
-                yield _(
-                    "Description has a list (%s) but it isn't bulleted (*) nor numbered (#)"
-                ) % lchar
-                break
-        else:
-            lchar = line[0]
-            lcount = 1
-
-
 def check_builds(app):
     supported_flags = set(metadata.build_flags)
     # needed for YAML and JSON
@@ -460,7 +439,7 @@ def check_builds(app):
                     "Build generated by `fdroid import` - remove disable line once ready"
                 )
             continue
-        for s in ['master', 'origin', 'HEAD', 'default', 'trunk']:
+        for s in ['master', 'main', 'origin', 'HEAD', 'default', 'trunk']:
             if build.commit and build.commit.startswith(s):
                 yield _(
                     "Branch '{branch}' used as commit in build '{versionName}'"
@@ -473,9 +452,12 @@ def check_builds(app):
                             "Branch '{branch}' used as commit in srclib '{srclib}'"
                         ).format(branch=s, srclib=srclib)
                 else:
-                    yield _(
-                        'srclibs missing name and/or @'
-                    ) + ' (srclibs: ' + srclib + ')'
+                    yield (
+                        _('srclibs missing name and/or @')
+                        + ' (srclibs: '
+                        + srclib
+                        + ')'
+                    )
         for key in build.keys():
             if key not in supported_flags:
                 yield _('%s is not an accepted build field') % key
@@ -514,7 +496,7 @@ def check_files_dir(app):
 
 
 def check_format(app):
-    if options.format and not rewritemeta.proper_format(app):
+    if common.options.format and not rewritemeta.proper_format(app):
         yield _("Run rewritemeta to fix formatting")
 
 
@@ -544,13 +526,23 @@ def check_extlib_dir(apps):
 
     used = set()
     for app in apps:
-        for build in app.get('Builds', []):
+        if app.Disabled:
+            continue
+        archive_policy = common.calculate_archive_policy(
+            app, common.config['archive_older']
+        )
+        builds = [build for build in app.Builds if not build.disable]
+
+        for i in range(len(builds)):
+            build = builds[i]
             for path in build.extlibs:
                 path = Path(path)
                 if path not in extlib_files:
-                    yield _(
-                        "{appid}: Unknown extlib {path} in build '{versionName}'"
-                    ).format(appid=app.id, path=path, versionName=build.versionName)
+                    # Don't show error on archived versions
+                    if i >= len(builds) - archive_policy:
+                        yield _(
+                            "{appid}: Unknown extlib {path} in build '{versionName}'"
+                        ).format(appid=app.id, path=path, versionName=build.versionName)
                 else:
                     used.add(path)
 
@@ -600,7 +592,7 @@ def check_app_field_types(app):
                     fieldtype=v.__class__.__name__,
                 )
             )
-        elif t == metadata.TYPE_STRING and not type(v) in (str, bool, dict):
+        elif t == metadata.TYPE_STRING and type(v) not in (str, bool, dict):
             yield (
                 _(
                     "{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!"
@@ -611,6 +603,48 @@ def check_app_field_types(app):
                     fieldtype=v.__class__.__name__,
                 )
             )
+        elif t == metadata.TYPE_STRINGMAP and not isinstance(v, dict):
+            yield (
+                _(
+                    "{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!"
+                ).format(
+                    appid=app.id,
+                    field=field,
+                    type='dict',
+                    fieldtype=v.__class__.__name__,
+                )
+            )
+        elif t == metadata.TYPE_INT and not isinstance(v, int):
+            yield (
+                _(
+                    "{appid}: {field} must be a '{type}', but it is a '{fieldtype}'!"
+                ).format(
+                    appid=app.id,
+                    field=field,
+                    type='int',
+                    fieldtype=v.__class__.__name__,
+                )
+            )
+
+
+def check_antiFeatures(app):
+    """Check the Anti-Features keys match those declared in the config."""
+    pattern = ANTIFEATURES_PATTERN
+    msg = _("'{value}' is not a valid {field} in {appid}. Regex pattern: {pattern}")
+
+    field = 'AntiFeatures'  # App entries use capitalized CamelCase
+    for value in app.get(field, []):
+        if value not in ANTIFEATURES_KEYS:
+            yield msg.format(value=value, field=field, appid=app.id, pattern=pattern)
+
+    field = 'antifeatures'  # Build entries use all lowercase
+    for build in app.get('Builds', []):
+        build_antiFeatures = build.get(field, [])
+        for value in build_antiFeatures:
+            if value not in ANTIFEATURES_KEYS:
+                yield msg.format(
+                    value=value, field=field, appid=app.id, pattern=pattern
+                )
 
 
 def check_for_unsupported_metadata_files(basedir=""):
@@ -649,8 +683,7 @@ def check_for_unsupported_metadata_files(basedir=""):
 
 def check_current_version_code(app):
     """Check that the CurrentVersionCode is currently available."""
-    archive_policy = app.get('ArchivePolicy')
-    if archive_policy and archive_policy.split()[0] == "0":
+    if app.get('ArchivePolicy') == 0:
         return
     cv = app.get('CurrentVersionCode')
     if cv is not None and cv == 0:
@@ -680,13 +713,11 @@ def check_current_version_code(app):
 
 def check_updates_expected(app):
     """Check if update checking makes sense."""
-    if (
-        app.get('NoSourceSince') or app.get('ArchivePolicy') == '0 versions'
-    ) and not all(
+    if (app.get('NoSourceSince') or app.get('ArchivePolicy') == 0) and not all(
         app.get(key, 'None') == 'None' for key in ('AutoUpdateMode', 'UpdateCheckMode')
     ):
         yield _(
-            'App has NoSourceSince or ArchivePolicy "0 versions" but AutoUpdateMode or UpdateCheckMode are not None'
+            'App has NoSourceSince or ArchivePolicy "0 versions" or 0 but AutoUpdateMode or UpdateCheckMode are not None'
         )
 
 
@@ -696,9 +727,104 @@ def check_updates_ucm_http_aum_pattern(app):  # noqa: D403
         yield _("AutoUpdateMode with UpdateCheckMode: HTTP must have a pattern.")
 
 
-def main():
+def check_certificate_pinned_binaries(app):
+    keys = app.get('AllowedAPKSigningKeys')
+    known_keys = common.config.get('apk_signing_key_block_list', [])
+    if keys:
+        if known_keys:
+            for key in keys:
+                if key in known_keys:
+                    yield _('Known debug key is used in AllowedAPKSigningKeys: ') + key
+        return
+    if app.get('Binaries') is not None:
+        yield _(
+            'App has Binaries but does not have corresponding AllowedAPKSigningKeys to pin certificate.'
+        )
+        return
+    builds = app.get('Builds')
+    if builds is None:
+        return
+    for build in builds:
+        if build.get('binary') is not None:
+            yield _(
+                'App version has binary but does not have corresponding AllowedAPKSigningKeys to pin certificate.'
+            )
+            return
 
-    global config, options
+
+def lint_config(arg):
+    path = Path(arg)
+    passed = True
+
+    mirrors_name = f'{common.MIRRORS_CONFIG_NAME}.yml'
+    config_name = f'{common.CONFIG_CONFIG_NAME}.yml'
+    categories_name = f'{common.CATEGORIES_CONFIG_NAME}.yml'
+    antifeatures_name = f'{common.ANTIFEATURES_CONFIG_NAME}.yml'
+
+    yamllintresult = common.run_yamllint(path)
+    if yamllintresult:
+        print(yamllintresult)
+        passed = False
+
+    with path.open() as fp:
+        data = ruamel.yaml.YAML(typ='safe').load(fp)
+    common.config_type_check(arg, data)
+
+    if path.name == mirrors_name:
+        import pycountry
+
+        valid_country_codes = [c.alpha_2 for c in pycountry.countries]
+        for mirror in data:
+            code = mirror.get('countryCode')
+            if code and code not in valid_country_codes:
+                passed = False
+                msg = _(
+                    '{path}: "{code}" is not a valid ISO_3166-1 alpha-2 country code!'
+                ).format(path=str(path), code=code)
+                if code.upper() in valid_country_codes:
+                    m = [code.upper()]
+                else:
+                    m = difflib.get_close_matches(
+                        code.upper(), valid_country_codes, 2, 0.5
+                    )
+                if m:
+                    msg += ' '
+                    msg += _('Did you mean {code}?').format(code=', '.join(sorted(m)))
+                print(msg)
+    elif path.name in (config_name, categories_name, antifeatures_name):
+        for key in data:
+            if path.name == config_name and key not in ('archive', 'repo'):
+                passed = False
+                print(
+                    _('ERROR: {key} in {path} is not "archive" or "repo"!').format(
+                        key=key, path=path
+                    )
+                )
+            allowed_keys = ['name']
+            if path.name in [config_name, antifeatures_name]:
+                allowed_keys.append('description')
+            # only for source strings currently
+            if path.parent.name == 'config':
+                allowed_keys.append('icon')
+            for subkey in data[key]:
+                if subkey not in allowed_keys:
+                    passed = False
+                    print(
+                        _(
+                            'ERROR: {key}:{subkey} in {path} is not in allowed keys: {allowed_keys}!'
+                        ).format(
+                            key=key,
+                            subkey=subkey,
+                            path=path,
+                            allowed_keys=', '.join(allowed_keys),
+                        )
+                    )
+
+    return passed
+
+
+def main():
+    global config
 
     # Parse command line...
     parser = ArgumentParser()
@@ -723,14 +849,46 @@ def main():
         "appid", nargs='*', help=_("application ID of file to operate on")
     )
     metadata.add_metadata_arguments(parser)
-    options = parser.parse_args()
+    options = common.parse_args(parser)
     metadata.warnings_action = options.W
 
-    config = common.read_config(options)
+    config = common.read_config()
+    load_antiFeatures_config()
+    load_categories_config()
 
-    # Get all apps...
-    allapps = metadata.read_metadata(options.appid)
-    apps = common.read_app_args(options.appid, allapps, False)
+    if options.force_yamllint:
+        import yamllint  # throw error if it is not installed
+
+        yamllint  # make pyflakes ignore this
+
+    paths = list()
+    for arg in options.appid:
+        if (
+            arg == 'config.yml'
+            or Path(arg).parent.name == 'config'
+            or Path(arg).parent.parent.name == 'config'  # localized
+        ):
+            paths.append(arg)
+
+    failed = 0
+    if paths:
+        for path in paths:
+            options.appid.remove(path)
+            if not lint_config(path):
+                failed += 1
+        # an empty list of appids means check all apps, avoid that if files were given
+        if not options.appid:
+            sys.exit(failed)
+
+    if not lint_metadata(options):
+        failed += 1
+
+    if failed:
+        sys.exit(failed)
+
+
+def lint_metadata(options):
+    apps = common.read_app_args(options.appid)
 
     anywarns = check_for_unsupported_metadata_files()
 
@@ -747,14 +905,8 @@ def main():
         if app.Disabled:
             continue
 
-        if options.force_yamllint:
-            import yamllint  # throw error if it is not installed
-
-            yamllint  # make pyflakes ignore this
-
         # only run yamllint when linting individual apps.
         if options.appid or options.force_yamllint:
-
             # run yamllint on app metadata
             ymlpath = Path('metadata') / (appid + '.yml')
             if ymlpath.is_file():
@@ -784,6 +936,7 @@ def main():
 
         app_check_funcs = [
             check_app_field_types,
+            check_antiFeatures,
             check_regexes,
             check_update_check_data_url,
             check_update_check_data_int,
@@ -797,7 +950,6 @@ def main():
             check_categories,
             check_duplicates,
             check_mediawiki_links,
-            check_bulleted_lists,
             check_builds,
             check_files_dir,
             check_format,
@@ -805,6 +957,7 @@ def main():
             check_current_version_code,
             check_updates_expected,
             check_updates_ucm_http_aum_pattern,
+            check_certificate_pinned_binaries,
         ]
 
         for check_func in app_check_funcs:
@@ -812,8 +965,7 @@ def main():
                 anywarns = True
                 print("%s: %s" % (appid, warn))
 
-    if anywarns:
-        sys.exit(1)
+    return not anywarns
 
 
 # A compiled, public domain list of official SPDX license tags.  generated
