@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+"""Set up an app build for a nightly build repo."""
 #
 # nightly.py - part of the FDroid server tools
 # Copyright (C) 2017 Hans-Christoph Steiner <hans@eds.org>
@@ -32,6 +33,7 @@ import tempfile
 import yaml
 from urllib.parse import urlparse
 from argparse import ArgumentParser
+from typing import Optional
 
 from . import _
 from . import common
@@ -48,12 +50,38 @@ DISTINGUISHED_NAME = 'CN=Android Debug,O=Android,C=US'
 NIGHTLY = '-nightly'
 
 
-def _get_keystore_secret_var(keystore):
+def _get_keystore_secret_var(keystore: str) -> str:
+    """Get keystore secret as base64.
+
+    Parameters
+    ----------
+    keystore
+        The path of the keystore.
+
+    Returns
+    -------
+    base64_secret
+        The keystore secret as base64 string.
+    """
     with open(keystore, 'rb') as fp:
         return base64.standard_b64encode(fp.read()).decode('ascii')
 
 
-def _ssh_key_from_debug_keystore(keystore=None):
+def _ssh_key_from_debug_keystore(keystore: Optional[str] = None) -> str:
+    """Convert a debug keystore to an SSH private key.
+
+    This leaves the original keystore file in place.
+
+    Parameters
+    ----------
+    keystore
+        The keystore to convert to a SSH private key.
+
+    Returns
+    -------
+    key_path
+        The SSH private key file path in the temporary directory.
+    """
     if keystore is None:
         # set this here so it can be overridden in the tests
         # TODO convert this to a class to get rid of this nonsense
@@ -148,7 +176,23 @@ def _ssh_key_from_debug_keystore(keystore=None):
     return ssh_private_key_file
 
 
-def get_repo_base_url(clone_url, repo_git_base, force_type=None):
+def get_repo_base_url(clone_url: str, repo_git_base: str, force_type: Optional[str] = None) -> str:
+    """Generate the base URL for the F-Droid repository.
+
+    Parameters
+    ----------
+    clone_url
+        The URL to clone the Git repository.
+    repo_git_base
+        The project path of the Git repository at the Git forge.
+    force_type
+        The Git forge of the project.
+
+    Returns
+    -------
+    repo_base_url
+        The base URL of the F-Droid repository.
+    """
     if force_type is None:
         force_type = urlparse(clone_url).netloc
     if force_type == 'gitlab.com':
@@ -160,7 +204,17 @@ def get_repo_base_url(clone_url, repo_git_base, force_type=None):
 
 
 def main():
+    """Deploy to F-Droid repository or generate SSH private key from keystore.
 
+    The behaviour of this function is influenced by the configuration file as
+    well as command line parameters.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.VCSException`
+        If the nightly Git repository could not be cloned during an attempt to
+        deploy.
+    """
     parser = ArgumentParser()
     common.setup_global_opts(parser)
     parser.add_argument(
@@ -205,8 +259,7 @@ def main():
         help=_("Set maximum releases in repo before older ones are archived"),
     )
     # TODO add --with-btlog
-    options = parser.parse_args()
-    common.options = options
+    options = common.parse_args(parser)
 
     # force a tighter umask since this writes private key material
     umask = os.umask(0o077)
@@ -307,11 +360,14 @@ def main():
         readme = '''
 # {repo_git_base}
 
-[![{repo_url}]({repo_url}/icons/icon.png)]({repo_url})
+This is an app repository for nightly versions.
+You can use it with the [F-Droid](https://f-droid.org/) Android app.
+
+[![{repo_url}]({repo_url}/icons/icon.png)](https://fdroid.link/#{repo_url})
 
 Last updated: {date}'''.format(repo_git_base=repo_git_base,
                                repo_url=repo_url,
-                               date=datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S UTC'))
+                               date=datetime.datetime.now(datetime.timezone.utc).strftime('%Y-%m-%d %H:%M:%S UTC'))
         with open(readme_path, 'w') as fp:
             fp.write(readme)
         mirror_git_repo.git.add(all=True)
@@ -322,11 +378,11 @@ Last updated: {date}'''.format(repo_git_base=repo_git_base,
 
         os.chdir(repo_basedir)
         if os.path.isdir(git_mirror_repodir):
-            common.local_rsync(options, git_mirror_repodir + '/', 'repo/')
+            common.local_rsync(options, [git_mirror_repodir + '/'], 'repo/')
         if os.path.isdir(git_mirror_metadatadir):
-            common.local_rsync(options, git_mirror_metadatadir + '/', 'metadata/')
+            common.local_rsync(options, [git_mirror_metadatadir + '/'], 'metadata/')
         if os.path.isdir(git_mirror_statsdir):
-            common.local_rsync(options, git_mirror_statsdir + '/', 'stats/')
+            common.local_rsync(options, [git_mirror_statsdir + '/'], 'stats/')
 
         ssh_private_key_file = _ssh_key_from_debug_keystore()
         # this is needed for GitPython to find the SSH key
@@ -359,25 +415,34 @@ Last updated: {date}'''.format(repo_git_base=repo_git_base,
             'archive_url': repo_base + '/archive',
             'archive_description': 'Old nightly builds that have been archived.',
             'archive_older': options.archive_older,
-            'servergitmirrors': servergitmirror,
+            'servergitmirrors': [{"url": servergitmirror}],
             'keystore': KEYSTORE_FILE,
             'repo_keyalias': KEY_ALIAS,
             'keystorepass': PASSWORD,
             'keypass': PASSWORD,
             'keydname': DISTINGUISHED_NAME,
             'make_current_version_link': False,
-            'update_stats': True,
         }
         with open('config.yml', 'w') as fp:
             yaml.dump(config, fp, default_flow_style=False)
         os.chmod('config.yml', 0o600)
-        config = common.read_config(options)
+        config = common.read_config()
         common.assert_config_keystore(config)
 
+        logging.debug(
+            _('Run over {cibase} to find -debug.apk. and skip repo_basedir {repo_basedir}').format(
+                cibase=cibase,
+                repo_basedir=repo_basedir
+            )
+        )
+
         for root, dirs, files in os.walk(cibase):
-            for d in dirs:
-                if d == '.git' or d == '.gradle' or (d == 'fdroid' and root == cibase):
+            for d in ('.git', '.gradle'):
+                if d in dirs:
                     dirs.remove(d)
+            if root == cibase and 'fdroid' in dirs:
+                dirs.remove('fdroid')
+
             for f in files:
                 if f.endswith('-debug.apk'):
                     apkfilename = os.path.join(root, f)
@@ -428,9 +493,11 @@ Last updated: {date}'''.format(repo_git_base=repo_git_base,
             cwd=repo_basedir,
         )
         common.local_rsync(
-            options, repo_basedir + '/metadata/', git_mirror_metadatadir + '/'
+            options, [repo_basedir + '/metadata/'], git_mirror_metadatadir + '/'
         )
-        common.local_rsync(options, repo_basedir + '/stats/', git_mirror_statsdir + '/')
+        stats = repo_basedir + '/stats/'
+        if os.path.exists(stats):
+            common.local_rsync(options, [stats], git_mirror_statsdir + '/')
         mirror_git_repo.git.add(all=True)
         mirror_git_repo.index.commit("update app metadata")
 
