@@ -18,12 +18,21 @@
 import os
 import sys
 import tempfile
+import unittest
+import unittest.mock
+
+from pathlib import Path
 
 
-class TmpCwd():
-    """Context-manager for temporarily changing the current working
-    directory.
-    """
+GP_FINGERPRINT = 'B7C2EEFD8DAC7806AF67DFCD92EB18126BC08312A7F2D6F3862E46013C7A6135'
+
+
+class VerboseFalseOptions:
+    verbose = False
+
+
+class TmpCwd:
+    """Context-manager for temporarily changing the current working directory."""
 
     def __init__(self, new_cwd):
         self.new_cwd = new_cwd
@@ -36,9 +45,8 @@ class TmpCwd():
         os.chdir(self.orig_cwd)
 
 
-class TmpPyPath():
-    """Context-manager for temporarily adding a direcory to python path
-    """
+class TmpPyPath:
+    """Context-manager for temporarily adding a directory to Python path."""
 
     def __init__(self, additional_path):
         self.additional_path = additional_path
@@ -51,14 +59,11 @@ class TmpPyPath():
 
 
 def mock_open_to_str(mock):
-    """
-    helper function for accessing all data written into a
-    unittest.mock.mock_open() instance as a string.
-    """
+    """For accessing all data written into a unittest.mock.mock_open() instance as a string."""
 
-    return "".join([
-        x.args[0] for x in mock.mock_calls if str(x).startswith("call().write(")
-    ])
+    return "".join(
+        [x.args[0] for x in mock.mock_calls if str(x).startswith("call().write(")]
+    )
 
 
 def mkdtemp():
@@ -66,3 +71,20 @@ def mkdtemp():
         return tempfile.TemporaryDirectory()
     else:
         return tempfile.TemporaryDirectory(ignore_cleanup_errors=True)
+
+
+def mkdir_testfiles(localmodule, test):
+    """Keep the test files in a labeled test dir for easy reference"""
+    testroot = Path(localmodule) / '.testfiles'
+    testroot.mkdir(exist_ok=True)
+    testdir = testroot / unittest.TestCase.id(test)
+    testdir.mkdir(exist_ok=True)
+    return tempfile.mkdtemp(dir=testdir)
+
+
+def mock_urlopen(status=200, body=None):
+    resp = unittest.mock.MagicMock()
+    resp.getcode.return_value = status
+    resp.read.return_value = body
+    resp.__enter__.return_value = resp
+    return unittest.mock.Mock(return_value=resp)
